/******************************************************************************
 *    AIScriptService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IAIScrSrv>::Name = "AIScriptService";

	const char * 
	ScriptService<IAIScrSrv>::DocStrings[] = {
		"AIScriptService()\n\t"
			"Interface for controlling AI objects.",
		"MakeGotoObjLoc(ai,obj,speed,priority,data=None) -> Boolean\n\t"
			"Make the ai go to obj. data will be returned in action result\n\t"
			"message. speed and priority are integers.",
		"MakeFrobObj2(ai,obj,with,priority,data) -> Boolean\n\t"
			"The ai will frob obj using with. data will be returned in action\n\t"
			"result message. with is an object. priority is an integer.",
		"MakeFrobObj(ai,obj,priority,data) -> Boolean\n\t"
			"The ai will frob obj. data will be returned in action result\n\t"
			"message. priority is an integer.",
		"GetAlertLevel(ai) -> Integer\n\t"
			"Returns the current alert level of ai.",
		"SetMinimumAlert(ai,level)\n\t"
			"Limits the alert level of ai to be no lower than level.\n\t"
			"The current alert level will change if it is lower than level.",
		"ClearGoals(ai)\n\t"
			"Causes ai to forget the actions it is engaged in.",
		"SetScriptFlags(ai,flags)\n\t"
			"I am unsure of what the script flags are. Thief 1/G used them\n\t"
			"a lot. Thief 2 not so much.",
		"ClearAlertness(ai)\n\t"
			"Reverts ai to its minimum alert level.",
		"Signal(ai,msg)\n\t"
			"Send a SignalAI message to ai. msg is a string.",
		"StartConversation(obj) -> Boolean\n\t"
			"Run the conversation defined by obj.",
	};

	const PyMethodDef 
	ScriptService<IAIScrSrv>::Methods[] = {
		{	"MakeGotoObjLoc",
			reinterpret_cast<PyCFunction>(MakeGotoObjLoc),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"MakeFrobObj2",
			reinterpret_cast<PyCFunction>(MakeFrobObj2),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"MakeFrobObj",
			reinterpret_cast<PyCFunction>(MakeFrobObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"GetAlertLevel",
			reinterpret_cast<PyCFunction>(GetAlertLevel),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"SetMinimumAlert",
			reinterpret_cast<PyCFunction>(SetMinimumAlert),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"ClearGoals",
			reinterpret_cast<PyCFunction>(ClearGoals),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"SetScriptFlags",
			reinterpret_cast<PyCFunction>(SetScriptFlags),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"ClearAlertness",
			reinterpret_cast<PyCFunction>(ClearAlertness),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"Signal",
			reinterpret_cast<PyCFunction>(Signal),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"StartConversation",
			reinterpret_cast<PyCFunction>(StartConversation),
			METH_VARARGS,
			const_cast<char*>(DocStrings[10])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IAIScrSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IAIScrSrv>::Name),
		sizeof(ScriptServiceObject<IAIScrSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IAIScrSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IAIScrSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IAIScrSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kNoAlert", kNoAlert);
		PyModule_AddIntConstant(module, "kLowAlert", kLowAlert);
		PyModule_AddIntConstant(module, "kModerateAlert", kModerateAlert);
		PyModule_AddIntConstant(module, "kHighAlert", kHighAlert);
		PyModule_AddIntConstant(module, "kSlow", kSlow);
		PyModule_AddIntConstant(module, "kNormalSpeed", kNormalSpeed);
		PyModule_AddIntConstant(module, "kFast", kFast);
		PyModule_AddIntConstant(module, "kLowPriorityAction", kLowPriorityAction);
		PyModule_AddIntConstant(module, "kNormalPriorityAction", kNormalPriorityAction);
		PyModule_AddIntConstant(module, "kHighPriorityAction", kHighPriorityAction);
	}

	ScriptServiceObject<IAIScrSrv> * 
	ScriptService<IAIScrSrv>::NewObject(PyObject *)
	{
		IAIScrSrv *iface = static_cast<IAIScrSrv*>(g_pScriptManager->GetService(IID_IAIScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IAIScrSrv> *obj = PyObject_New(ScriptServiceObject<IAIScrSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::MakeGotoObjLoc(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		eAIActionPriority priority;
		eAIScriptSpeed speed;
		object ai, obj;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&O&ii|O&:MakeGotoObjLoc", cvtObject, &ai, cvtObject, &obj, &speed, &priority, cvtMultiParm, &data))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->MakeGotoObjLoc(ret,ai,obj,speed,priority,data);
		data.Free();
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::MakeFrobObj2(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		eAIActionPriority priority;
		object ai, obj, with;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&O&O&iO&:MakeFrobObj2", cvtObject, &ai, cvtObject, &obj, cvtObject, &with, &priority, cvtMultiParm, &data))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->MakeFrobObj(ret,ai,obj,with,priority,data);
		data.Free();
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::MakeFrobObj(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		eAIActionPriority priority;
		object ai, obj;
		cMultiParm data;
		PyObject *obData;
		if (!PyArg_ParseTuple(args, "O&O&iO&:MakeFrobObj", cvtObject, &ai, cvtObject, &obj, &priority, cvtMultiParm, &obData))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->MakeFrobObj(ret,ai,obj,priority,data);
		data.Free();
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::GetAlertLevel(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		object ai;
		if (!PyArg_ParseTuple(args, "O&:GetAlertLevel", cvtObject, &ai))
		{
			return NULL;
		}
		int level = self->m_pIFace->GetAlertLevel(ai);
		return PyInt_FromLong(level);
	}

	PyObject * 
	ScriptService<IAIScrSrv>::SetMinimumAlert(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		eAIScriptAlertLevel level;
		object ai;
		if (!PyArg_ParseTuple(args, "O&i:SetMinimumAlert", cvtObject, &ai, &level))
		{
			return NULL;
		}
		self->m_pIFace->SetMinimumAlert(ai,level);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::ClearGoals(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		object ai;
		if (!PyArg_ParseTuple(args, "O&:ClearGoals", cvtObject, &ai))
		{
			return NULL;
		}
		self->m_pIFace->ClearGoals(ai);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::SetScriptFlags(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		int flags;
		object ai;
		if (!PyArg_ParseTuple(args, "O&i:SetScriptFlags", cvtObject, &ai, &flags))
		{
			return NULL;
		}
		self->m_pIFace->SetScriptFlags(ai,flags);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::ClearAlertness(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		object ai;
		if (!PyArg_ParseTuple(args, "O&:ClearAlertness", cvtObject, &ai))
		{
			return NULL;
		}
		self->m_pIFace->ClearAlertness(ai);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::Signal(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		char const * msg;
		object ai;
		if (!PyArg_ParseTuple(args, "O&s:Signal", cvtObject, &ai, &msg))
		{
			return NULL;
		}
		self->m_pIFace->Signal(ai,msg);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IAIScrSrv>::StartConversation(ScriptServiceObject<IAIScrSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:StartConversation", cvtObject, &obj))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->StartConversation(ret,obj);
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}


} // namespace PyNumbra


