/******************************************************************************
 *    DarkInvService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IDarkInvSrv>::Name = "DarkInvService";

	const char * 
	ScriptService<IDarkInvSrv>::DocStrings[] = {
		"DarkInvService()\n\t"
			"I'm not actually sure what \"Inv\" refers to. It could be\n\t"
			"\"Inventory\", but what does that have to do with speed potions?",
		"CapabilityControl(cap,ctrl)\n\t"
			"Arguments are integers. (Not that I've been able to control\n\t"
			"squat with this function.)",
		"AddSpeedControl(name,scale,rot)\n\t"
			"name is a string. The other arguments are floats.\n\t"
			"rot is supposed to scale turning speed, but it doesn't seem\n\t"
			"to be effective.",
		"RemoveSpeedControl(name)\n\t"
			"Use the same name supplied to AddSpeedControl, obviously.",
	};

	const PyMethodDef 
	ScriptService<IDarkInvSrv>::Methods[] = {
		{	"CapabilityControl",
			reinterpret_cast<PyCFunction>(CapabilityControl),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"AddSpeedControl",
			reinterpret_cast<PyCFunction>(AddSpeedControl),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"RemoveSpeedControl",
			reinterpret_cast<PyCFunction>(RemoveSpeedControl),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDarkInvSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDarkInvSrv>::Name),
		sizeof(ScriptServiceObject<IDarkInvSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDarkInvSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDarkInvSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IDarkInvSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kDrkInvCapCycle", kDrkInvCapCycle);
		PyModule_AddIntConstant(module, "kDrkInvCapWorldFrob", kDrkInvCapWorldFrob);
		PyModule_AddIntConstant(module, "kDrkInvCapWorldFocus", kDrkInvCapWorldFocus);
		PyModule_AddIntConstant(module, "kDrkInvCapInvFrob", kDrkInvCapInvFrob);
		PyModule_AddIntConstant(module, "kDrkInvControlOn", kDrkInvControlOn);
		PyModule_AddIntConstant(module, "kDrkInvControlOff", kDrkInvControlOff);
		PyModule_AddIntConstant(module, "kDrkInvControlToggle", kDrkInvControlToggle);
	}

	ScriptServiceObject<IDarkInvSrv> * 
	ScriptService<IDarkInvSrv>::NewObject(PyObject *)
	{
		IDarkInvSrv *iface = static_cast<IDarkInvSrv*>(g_pScriptManager->GetService(IID_IDrkInvScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDarkInvSrv> *obj = PyObject_New(ScriptServiceObject<IDarkInvSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDarkInvSrv>::CapabilityControl(ScriptServiceObject<IDarkInvSrv> *self, PyObject *args)
	{
		eDrkInvCap cap;
		eDrkInvControl ctrl;
		if (!PyArg_ParseTuple(args, "ii:CapabilityControl", &cap, &ctrl))
		{
			return NULL;
		}
		self->m_pIFace->CapabilityControl(cap,ctrl);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IDarkInvSrv>::AddSpeedControl(ScriptServiceObject<IDarkInvSrv> *self, PyObject *args)
	{
		char *name;
		float low, high;
		if (!PyArg_ParseTuple(args, "sff:AddSpeedControl", &name, &low, &high))
		{
			return NULL;
		}
		self->m_pIFace->AddSpeedControl(name,low,high);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IDarkInvSrv>::RemoveSpeedControl(ScriptServiceObject<IDarkInvSrv> *self, PyObject *args)
	{
		char *name;
		if (!PyArg_ParseTuple(args, "s:RemoveSpeedControl", &name))
		{
			return NULL;
		}
		self->m_pIFace->RemoveSpeedControl(name);
		Py_INCREF(Py_None);
		return Py_None;
	}


} // namespace PyNumbra


