/******************************************************************************
 *    DarkPowerupsService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IDarkPowerupsSrv>::Name = "DarkPowerupsService";

	const char * 
	ScriptService<IDarkPowerupsSrv>::DocStrings[] = {
		"DarkPowerupsService()\n\t"
			"Various functions, mostly dealing with objects that the player\n\t"
			"throws or shoots.",
		"TriggerWorldFlash(obj)\n\t"
			"The 'SFX\\FlashBombInfo' property of the object will be used to\n\t"
			"generate a flash SFX at its location. It also stuns any AI that\n\t"
			"happen to have a line-of-sight to the object.",
		"ObjTryDeploy(obj1,obj2) -> Integer\n\t"
			"Creates an object hosted by another object.",
		"CleanseBlood(obj,radius)\n\t"
			"These taffers have no respect for such beautiful things...\n\t"
			"Any objects the game considers as blood that are within\n\t"
			"the radius centered on the obj will be slain.",
	};

	const PyMethodDef 
	ScriptService<IDarkPowerupsSrv>::Methods[] = {
		{	"TriggerWorldFlash",
			reinterpret_cast<PyCFunction>(TriggerWorldFlash),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"ObjTryDeploy",
			reinterpret_cast<PyCFunction>(ObjTryDeploy),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"CleanseBlood",
			reinterpret_cast<PyCFunction>(CleanseBlood),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDarkPowerupsSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDarkPowerupsSrv>::Name),
		sizeof(ScriptServiceObject<IDarkPowerupsSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDarkPowerupsSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDarkPowerupsSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IDarkPowerupsSrv> * 
	ScriptService<IDarkPowerupsSrv>::NewObject(PyObject *)
	{
		IDarkPowerupsSrv *iface = static_cast<IDarkPowerupsSrv*>(g_pScriptManager->GetService(IID_IDrkPowerupsScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDarkPowerupsSrv> *obj = PyObject_New(ScriptServiceObject<IDarkPowerupsSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDarkPowerupsSrv>::TriggerWorldFlash(ScriptServiceObject<IDarkPowerupsSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:TriggerWorldFlash", cvtObject, &obj))
		{
			return NULL;
		}
		self->m_pIFace->TriggerWorldFlash(obj);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IDarkPowerupsSrv>::ObjTryDeploy(ScriptServiceObject<IDarkPowerupsSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:ObjTryDeploy", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		int ret = self->m_pIFace->ObjTryDeploy(obj1,obj2); // XXX: I think this is supposed to be non-zero on success
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkPowerupsSrv>::CleanseBlood(ScriptServiceObject<IDarkPowerupsSrv> *self, PyObject *args)
	{
		float radius;
		object obj;
		if (!PyArg_ParseTuple(args, "O&f:CleanseBlood", cvtObject, &obj, &radius))
		{
			return NULL;
		}
		self->m_pIFace->CleanseBlood(obj,radius);
		Py_INCREF(Py_None);
		return Py_None;
	}


} // namespace PyNumbra


