/******************************************************************************
 *    DarkUIService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<IDarkUISrv>::Name = "DarkUIService";

	const char * 
	ScriptService<IDarkUISrv>::DocStrings[] = {
		"DarkUIService()\n\t"
			"Functions related to the in-game interface the player sees.",
		"TextMessage(msg,rgb=0,time=-1001) -> Integer\n\t"
			"rgb can be either an integer triple, or a single integer\n\t"
			"equivalent to the Windows COLORREF type. time is specified\n\t"
			"in milliseconds.\n\t"
			"The default color is white. The default time is about 5 seconds.",
		"ReadBook(book,art) -> Integer\n\t"
			"If either resource doesn't exist, the game will crash.",
		"InvItem() -> PyNumbra.Object\n\t"
			"The currently selected inventory item.",
		"InvWeapon() -> PyNumbra.Object\n\t"
			"The currently selected weapon.",
		"InvSelect(obj) -> Integer\n\t"
			"I have no idea what happens if the object isn't in the player's\n\t"
			"inventory.",
		"IsCommandBound(name) -> Boolean\n\t"
			"Checks if there is a key binding for the command.",
		"DescribeKeyBinding(name) -> String\n\t"
			"Returns a string representation of the keys that are bound to the\n\t"
			"given command.",
	};

	const PyMethodDef 
	ScriptService<IDarkUISrv>::Methods[] = {
		{	"TextMessage",
			reinterpret_cast<PyCFunction>(TextMessage),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"ReadBook",
			reinterpret_cast<PyCFunction>(ReadBook),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"InvItem",
			reinterpret_cast<PyCFunction>(InvItem),
			METH_NOARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"InvWeapon",
			reinterpret_cast<PyCFunction>(InvWeapon),
			METH_NOARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"InvSelect",
			reinterpret_cast<PyCFunction>(InvSelect),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"IsCommandBound",
			reinterpret_cast<PyCFunction>(IsCommandBound),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"DescribeKeyBinding",
			reinterpret_cast<PyCFunction>(DescribeKeyBinding),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDarkUISrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDarkUISrv>::Name),
		sizeof(ScriptServiceObject<IDarkUISrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDarkUISrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDarkUISrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IDarkUISrv> * 
	ScriptService<IDarkUISrv>::NewObject(PyObject *)
	{
		IDarkUISrv *iface = static_cast<IDarkUISrv*>(g_pScriptManager->GetService(IID_IDarkUIScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDarkUISrv> *obj = PyObject_New(ScriptServiceObject<IDarkUISrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDarkUISrv>::TextMessage(ScriptServiceObject<IDarkUISrv> *self, PyObject *args)
	{
		COLORREF rgb = RGB(0,0,0);
		int time = -1001;
		const char *msg;
		PyObject *obColor = NULL;
		if (!PyArg_ParseTuple(args, "s|Oi:TextMessage", &msg, &obColor, &time))
		{
			return NULL;
		}
		if (obColor)
		{
			if (PySequence_Check(obColor))
			{
				char r,g,b;
				if (PySequence_Size(obColor) != 3)
				{
					PyErr_SetString(PyExc_ValueError, "color must be a 3-tuple or integer");
					return NULL;
				}
				PyObject *tuple = PySequence_Tuple(obColor);
				if (!PyArg_ParseTuple(tuple, "bbb", &r, &g, &b))
				{
					Py_DECREF(tuple);
					PyErr_SetString(PyExc_ValueError, "color must be a 3-tuple of integers");
					return NULL;
				}
				rgb = RGB(r,g,b);
				Py_DECREF(tuple);
			}
			else
			{
				rgb = PyInt_AsLong(obColor);
				if (PyErr_Occurred())
				{
					PyErr_SetString(PyExc_ValueError, "color must be a 3-tuple or integer");
					return NULL;
				}
			}
		}
		long ret = self->m_pIFace->TextMessage(msg,rgb,time);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkUISrv>::ReadBook(ScriptServiceObject<IDarkUISrv> *self, PyObject *args)
	{
		const char *book, *art;
		if (!PyArg_ParseTuple(args, "ss:ReadBook", &book, &art))
		{
			return NULL;
		}
		long ret = self->m_pIFace->ReadBook(book,art);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkUISrv>::InvItem(ScriptServiceObject<IDarkUISrv> *self)
	{
		object item;
		self->m_pIFace->InvItem(item);
		if (item == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(item);
	}

	PyObject * 
	ScriptService<IDarkUISrv>::InvWeapon(ScriptServiceObject<IDarkUISrv> *self)
	{
		object item;
		self->m_pIFace->InvWeapon(item);
		if (item == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(item);
	}

	PyObject * 
	ScriptService<IDarkUISrv>::InvSelect(ScriptServiceObject<IDarkUISrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:InvSelect", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->InvSelect(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDarkUISrv>::IsCommandBound(ScriptServiceObject<IDarkUISrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:IsCommandBound", &name))
		{
			return NULL;
		}
		true_bool ret;
		self->m_pIFace->IsCommandBound(ret, name);
		PyObject *obRet = (ret) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<IDarkUISrv>::DescribeKeyBinding(ScriptServiceObject<IDarkUISrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:DescribeKeyBinding", &name))
		{
			return NULL;
		}
		cScrStr ret;
		self->m_pIFace->DescribeKeyBinding(ret, name);
		return PyString_FromString(static_cast<const char*>(ret));
	}


} // namespace PyNumbra


