/******************************************************************************
 *    DebugScriptService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <algorithm>
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

using std::min;

namespace PyNumbra
{

	const char *
	ScriptService<IDebugScrSrv>::Name = "DebugScriptService";

	const char * 
	ScriptService<IDebugScrSrv>::DocStrings[] = {
		"DebugScriptService()\n\t"
			"Convenient functions for finding out why your script isn't doing\n\t"
			"what you wanted to.",
		"MPrint(...) -> Integer\n\t"
			"Accepts up to eight strings which will be concatenated together\n\t"
			"and output to the monolog. The output will be terminated with a\n\t"
			"new-line.\n\t"
			"Actually, I cheat a little and allow you to pass more than eight\n\t"
			"arguments. However, if the total length of the output exceeds\n\t"
			"1022 characters, the game will crash. I make no attempts at\n\t"
			"protecting you from this bug, because you don't have that luxury\n\t"
			"when calling it from C++.\n\t"
			"If it's luxury you want, just print to stdout, which is tied to\n\t"
			"the monolog, and accounts for the 1022 byte mprintf limit.",
		"Command(...) -> Integer\n\t"
			"Accepts up to eight strings, with the same conditions and caveats\n\t"
			"of MPrint. The first argument is a command that will be executed,\n\t"
			"the rest will be concatenated and passed as arguments to the\n\t"
			"command. There will be a space between the command and its\n\t"
			"arguments, but not between the arguments themselves.",
		"Break() -> Integer\n\t"
			"Calls INT 3 software interrupt.\n\t"
			"Are you sure? (A)bort,(R)etry,(F)ail...",
	};

	const PyMethodDef 
	ScriptService<IDebugScrSrv>::Methods[] = {
		{	"MPrint",
			reinterpret_cast<PyCFunction>(MPrint),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"Command",
			reinterpret_cast<PyCFunction>(Command),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"Break",
			reinterpret_cast<PyCFunction>(Break),
			METH_NOARGS,
			const_cast<char*>(DocStrings[3])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IDebugScrSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IDebugScrSrv>::Name),
		sizeof(ScriptServiceObject<IDebugScrSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IDebugScrSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IDebugScrSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IDebugScrSrv> * 
	ScriptService<IDebugScrSrv>::NewObject(PyObject *)
	{
		IDebugScrSrv *iface = static_cast<IDebugScrSrv*>(g_pScriptManager->GetService(IID_IDebugScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IDebugScrSrv> *obj = PyObject_New(ScriptServiceObject<IDebugScrSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IDebugScrSrv>::MPrint(ScriptServiceObject<IDebugScrSrv> *self, PyObject *args)
	{
		PyObject * str[8];
		const int argc = PyTuple_GET_SIZE(args);
		int i = 0;
		unsigned int len = 0;
		for (; i < min(argc,8); ++i)
		{
			str[i] = PyObject_Str(PyTuple_GET_ITEM(args, i));
			if (!str[i])
			{
				while (i--)
				{
					Py_DECREF(str[i]);
				}
				PyErr_SetString(PyExc_TypeError, "MPrint accepts string arguments only.");
				return NULL;
			}
			len += PyString_GET_SIZE(str[i]);
		}
		if (argc > 8)
		{
			len -= PyString_GET_SIZE(str[7]);
			for (int j = i; j < argc; ++j)
			{
				PyString_ConcatAndDel(&str[7], PyObject_Str(PyTuple_GET_ITEM(args, j)));
				if (!str[7])
				{
					for (i = 6; i--; )
					{
						Py_DECREF(str[i]);
					}
					PyErr_SetString(PyExc_TypeError, "MPrint accepts string arguments only.");
					return NULL;
				}
			}
			len += PyString_GET_SIZE(str[7]);
		}
		if (i < 8)
		{
			PyObject *fill = PyString_FromString("");
			do
			{
				Py_INCREF(fill);
				str[i] = fill;
			}
			while (++i < 8);
			Py_DECREF(fill);
		}
		if (len > 1022)
		{
			self->m_pIFace->MPrint("*** WARNING: ",
				"mprintf string greater than 1022 characters.\n",
				"*** Stack will self-destruct in 5..4..3..2...",
				"","","","","");
			// No, I don't think I do want to do anything to avoid the inevitable crash.
			// We need to learn to avoid this bug.
			PyErr_Warn(PyExc_RuntimeWarning, "mprintf string greater than 1022 characters");
		}
		long ret = self->m_pIFace->MPrint(
			PyString_AS_STRING(str[0]),
			PyString_AS_STRING(str[1]),
			PyString_AS_STRING(str[2]),
			PyString_AS_STRING(str[3]),
			PyString_AS_STRING(str[4]),
			PyString_AS_STRING(str[5]),
			PyString_AS_STRING(str[6]),
			PyString_AS_STRING(str[7])
			);
		for (i = 8; i--; )
			Py_DECREF(str[i]);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDebugScrSrv>::Command(ScriptServiceObject<IDebugScrSrv> *self, PyObject *args)
	{
		PyObject * str[8];
		const int argc = PyTuple_GET_SIZE(args);
		int i = 0;
		unsigned int len = 0;
		for (; i < min(argc,8); ++i)
		{
			str[i] = PyObject_Str(PyTuple_GET_ITEM(args, i));
			if (!str[i])
			{
				while (i--)
				{
					Py_DECREF(str[i]);
				}
				return NULL;
			}
			len += PyString_GET_SIZE(str[i]);
		}
		if (argc > 8)
		{
			len -= PyString_GET_SIZE(str[7]);
			for (int j = i; j < argc; ++j)
			{
				PyString_ConcatAndDel(&str[7], PyObject_Str(PyTuple_GET_ITEM(args, j)));
				if (!str[7])
				{
					for (i = 6; i--; )
					{
						Py_DECREF(str[i]);
					}
					return NULL;
				}
			}
			len += PyString_GET_SIZE(str[7]);
		}
		if (i < 8)
		{
			PyObject *fill = PyString_FromString("");
			do
			{
				Py_INCREF(fill);
				str[i] = fill;
			}
			while (++i < 8);
			Py_DECREF(fill);
		}
		if (len > 1022)
		{
			self->m_pIFace->MPrint("*** WARNING: ",
				"mprintf string greater than 1022 characters.\n",
				"*** Stack will self-destruct in 5..4..3..2...",
				"","","","","");
			// No, I don't think I do want to do anything to avoid the inevitable crash.
			// We need to learn to avoid this bug.
			PyErr_Warn(PyExc_RuntimeWarning, "mprintf string greater than 1022 characters");
		}
		long ret = self->m_pIFace->Command(
			PyString_AS_STRING(str[0]),
			PyString_AS_STRING(str[1]),
			PyString_AS_STRING(str[2]),
			PyString_AS_STRING(str[3]),
			PyString_AS_STRING(str[4]),
			PyString_AS_STRING(str[5]),
			PyString_AS_STRING(str[6]),
			PyString_AS_STRING(str[7])
			);
		for (i = 8; i--; )
			Py_DECREF(str[i]);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IDebugScrSrv>::Break(ScriptServiceObject<IDebugScrSrv> *self)
	{
		return PyInt_FromLong(self->m_pIFace->Break());
	}


} // namespace PyNumbra


