/******************************************************************************
 *    ObjectService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraTypes.h"

namespace PyNumbra
{

	const char *
	ScriptService<IObjectSrv>::Name = "ObjectService";

	const char * 
	ScriptService<IObjectSrv>::DocStrings[] = {
		"ObjectService()\n\t"
			"Functions related to objects, the object hierarchy, and the\n\t"
			"object system. It would hardly be an \"object script system\"\n\t"
			"without this.",
		"BeginCreate(archetype) -> PyNumbra.Object\n\t"
			"Partially creates an object that inherits directly from the\n\t"
			"archetype. I'm not entirely sure how this differs from the\n\t"
			"atomic Create.",
		"EndCreate(obj) -> Integer\n\t"
			"Finalizes the initialization of the object.",
		"Create(archetype) -> PyNumbra.Object\n\t"
			"Adds a new object that inherits directly from the archetype.\n\t"
			"Or it will create a duplicate of a concrete object.",
		"Destroy(obj) -> Integer\n\t"
			"Removes the object from the database.",
		"Exists(obj) -> Boolean\n\t"
			"Tests if the object ID is valid.",
		"SetName(obj,name) -> Integer\n\t"
			"Sets the symbolic name of the object.",
		"GetName(obj) -> String\n\t"
			"Returns the symbolic name of the object.",
		"Named(name) -> PyNumbra.Object\n\t"
			"Returns the object with a given symbolic name.",
		"AddMetaProperty(obj,mp) -> Integer\n\t"
			"Adds mp to obj as a meta-property.",
		"RemoveMetaProperty(obj,mp) -> Integer\n\t"
			"Removes the association between an object and a meta-property.",
		"HasMetaProperty(obj,mp) -> Boolean\n\t"
			"Returns True if the meta-property object is associated with the\n\t"
			"object..",
		"InheritsFrom(obj,archetype) -> Boolean\n\t"
			"Tests if an object is a descendant of a particular archetype.",
		"IsTransient(obj) -> Boolean\n\t"
			"Transient objects are not saved in the mission database.",
		"SetTransience(obj,on) -> Integer\n\t"
			"Sets or clears the transient flag of an object.",
		"Position(obj) -> Vector\n\t"
			"Returns the current location of the object.",
		"Facing(obj) -> Vector\n\t"
			"Returns the current rotation of the object.",
		"Teleport(obj,location,rotation,relative=0) -> Integer\n\t"
			"Sets the location and rotation vectors for obj. If relative is\n\t"
			"given, the arguments are offsets relative to the relative\n\t"
			"object.",
		"IsPositionValid(obj) -> Boolean\n\t"
			"Only available in Thief 2 and System Shock 2.",
		"FindClosestObjectNamed(obj,name) -> PyNumbra.Object\n\t"
			"Only available in Thief 2.",
		"AddMetaPropertyToMany(mp,objs) -> Integer",
		"RemoveMetaPropertyFromMany(mp,objs) -> Integer",
		"RenderedThisFrame(obj) -> Boolean\n\t"
			"Checks if the object is currently visible on the screen.",
	};

	const PyMethodDef 
	ScriptService<IObjectSrv>::Methods[] = {
		{	"BeginCreate",
			reinterpret_cast<PyCFunction>(BeginCreate),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"EndCreate",
			reinterpret_cast<PyCFunction>(EndCreate),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"Create",
			reinterpret_cast<PyCFunction>(Create),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"Destroy",
			reinterpret_cast<PyCFunction>(Destroy),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"Exists",
			reinterpret_cast<PyCFunction>(Exists),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"SetName",
			reinterpret_cast<PyCFunction>(SetName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"GetName",
			reinterpret_cast<PyCFunction>(GetName),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"Named",
			reinterpret_cast<PyCFunction>(Named),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"AddMetaProperty",
			reinterpret_cast<PyCFunction>(AddMetaProperty),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"RemoveMetaProperty",
			reinterpret_cast<PyCFunction>(RemoveMetaProperty),
			METH_VARARGS,
			const_cast<char*>(DocStrings[10])
		},
		{	"HasMetaProperty",
			reinterpret_cast<PyCFunction>(HasMetaProperty),
			METH_VARARGS,
			const_cast<char*>(DocStrings[11])
		},
		{	"InheritsFrom",
			reinterpret_cast<PyCFunction>(InheritsFrom),
			METH_VARARGS,
			const_cast<char*>(DocStrings[12])
		},
		{	"IsTransient",
			reinterpret_cast<PyCFunction>(IsTransient),
			METH_VARARGS,
			const_cast<char*>(DocStrings[13])
		},
		{	"SetTransience",
			reinterpret_cast<PyCFunction>(SetTransience),
			METH_VARARGS,
			const_cast<char*>(DocStrings[14])
		},
		{	"Position",
			reinterpret_cast<PyCFunction>(Position),
			METH_VARARGS,
			const_cast<char*>(DocStrings[15])
		},
		{	"Facing",
			reinterpret_cast<PyCFunction>(Facing),
			METH_VARARGS,
			const_cast<char*>(DocStrings[16])
		},
		{	"Teleport",
			reinterpret_cast<PyCFunction>(Teleport),
			METH_VARARGS,
			const_cast<char*>(DocStrings[17])
		},
#if (_DARKGAME == 2) || (_DARKGAME == 3)
		{	"IsPositionValid",
			reinterpret_cast<PyCFunction>(IsPositionValid),
			METH_VARARGS,
			const_cast<char*>(DocStrings[18])
		},
#endif
#if (_DARKGAME == 2)
		{	"FindClosestObjectNamed",
			reinterpret_cast<PyCFunction>(FindClosestObjectNamed),
			METH_VARARGS,
			const_cast<char*>(DocStrings[19])
		},
#endif
		{	"AddMetaPropertyToMany",
			reinterpret_cast<PyCFunction>(AddMetaPropertyToMany),
			METH_VARARGS,
			const_cast<char*>(DocStrings[20])
		},
		{	"RemoveMetaPropertyFromMany",
			reinterpret_cast<PyCFunction>(RemoveMetaPropertyFromMany),
			METH_VARARGS,
			const_cast<char*>(DocStrings[21])
		},
		{	"RenderedThisFrame",
			reinterpret_cast<PyCFunction>(RenderedThisFrame),
			METH_VARARGS,
			const_cast<char*>(DocStrings[22])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IObjectSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IObjectSrv>::Name),
		sizeof(ScriptServiceObject<IObjectSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IObjectSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IObjectSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IObjectSrv> * 
	ScriptService<IObjectSrv>::NewObject(PyObject *)
	{
		IObjectSrv *iface = static_cast<IObjectSrv*>(g_pScriptManager->GetService(IID_IObjectScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IObjectSrv> *obj = PyObject_New(ScriptServiceObject<IObjectSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IObjectSrv>::BeginCreate(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object archetype;
		if (!PyArg_ParseTuple(args, "O&:BeginCreate", cvtObject, &archetype))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->BeginCreate(ret,archetype);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::EndCreate(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:EndCreate", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->EndCreate(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Create(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object archetype;
		if (!PyArg_ParseTuple(args, "O&:Create", cvtObject, &archetype))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->Create(ret,archetype);
		if (ret == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Destroy(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Destroy", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Destroy(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Exists(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Exists", cvtObject, &obj))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->Exists(p,obj);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IObjectSrv>::SetName(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:SetName", cvtObject, &obj, &name))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SetName(obj,name);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::GetName(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:GetName", cvtObject, &obj))
		{
			return NULL;
		}
		cScrStr name;
		self->m_pIFace->GetName(name,obj);
		return PyString_FromString(static_cast<const char*>(name));
	}

	PyObject * 
	ScriptService<IObjectSrv>::Named(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:Named", &name))
		{
			return NULL;
		}
		object obj;
		self->m_pIFace->Named(obj,name);
		if (obj == 0)
		{
			Py_INCREF(Py_None);
			return Py_None;
		}
		return ObjectObject::New(obj);
	}

	PyObject * 
	ScriptService<IObjectSrv>::AddMetaProperty(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj, mp;
		if (!PyArg_ParseTuple(args, "O&O&:AddMetaProperty", cvtObject, &obj, cvtObject, &mp))
		{
			return NULL;
		}
		long ret = self->m_pIFace->AddMetaProperty(obj,mp);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::RemoveMetaProperty(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj, mp;
		if (!PyArg_ParseTuple(args, "O&O&:RemoveMetaProperty", cvtObject, &obj, cvtObject, &mp))
		{
			return NULL;
		}
		long ret = self->m_pIFace->RemoveMetaProperty(obj,mp);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::HasMetaProperty(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj, mp;
		if (!PyArg_ParseTuple(args, "O&O&:HasMetaProperty", cvtObject, &obj, cvtObject, &mp))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->HasMetaProperty(p,obj,mp);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IObjectSrv>::InheritsFrom(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj, archetype;
		if (!PyArg_ParseTuple(args, "O&O&:InheritsFrom", cvtObject, &obj, cvtObject, &archetype))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->InheritsFrom(p,obj,archetype);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IObjectSrv>::IsTransient(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsTransient", cvtObject, &obj))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->IsTransient(p,obj);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IObjectSrv>::SetTransience(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		int on;
		object obj;
		if (!PyArg_ParseTuple(args, "O&i:SetTransience", cvtObject, &obj, &on))
		{
			return NULL;
		}
		long ret = self->m_pIFace->SetTransience(obj,on);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Position(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Position", cvtObject, &obj))
		{
			return NULL;
		}
		cScrVec v;
		self->m_pIFace->Position(v,obj);
		return Py_BuildValue("[fff]", v.pos.x, v.pos.y, v.pos.z);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Facing(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:Facing", cvtObject, &obj))
		{
			return NULL;
		}
		cScrVec v;
		self->m_pIFace->Facing(v,obj);
		return Py_BuildValue("[fff]", v.pos.x, v.pos.y, v.pos.z);
	}

	PyObject * 
	ScriptService<IObjectSrv>::Teleport(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		cScrVec loc, rot;
		object obj;
		object rel = 0;
		if (!PyArg_ParseTuple(args, "O&(fff)(fff)|O&:Teleport", cvtObject, &obj, 
							&loc.pos.x, &loc.pos.y, &loc.pos.z,
							&rot.pos.x, &rot.pos.y, &rot.pos.z, cvtObject, &rot))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Teleport(obj,loc,rot,rel);
		return PyInt_FromLong(ret);
	}

#if (_DARKGAME == 2) || (_DARKGAME == 3)
	PyObject * 
	ScriptService<IObjectSrv>::IsPositionValid(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsPositionValid", cvtObject, &obj))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->IsPositionValid(p,obj);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}
#endif

#if (_DARKGAME == 2)
	PyObject * 
	ScriptService<IObjectSrv>::FindClosestObjectNamed(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:FindClosestObjectNamed", cvtObject, &obj, &name))
		{
			return NULL;
		}
		object ret;
		self->m_pIFace->FindClosestObjectNamed(ret,obj,name);
		return ObjectObject::New(ret);
	}
#endif

	PyObject * 
	ScriptService<IObjectSrv>::AddMetaPropertyToMany(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		const char *objs;
		object mp;
		if (!PyArg_ParseTuple(args, "O&s:AddMetaPropertyToMany", cvtObject, &mp, &objs))
		{
			return NULL;
		}
		int ret = self->m_pIFace->AddMetaPropertyToMany(mp,objs);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::RemoveMetaPropertyFromMany(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		const char *objs;
		object mp;
		if (!PyArg_ParseTuple(args, "O&s:RemoveMetaPropertyFromMany", cvtObject, &mp, &objs))
		{
			return NULL;
		}
		int ret = self->m_pIFace->RemoveMetaPropertyFromMany(mp,objs);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IObjectSrv>::RenderedThisFrame(ScriptServiceObject<IObjectSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:RenderedThisFrame", cvtObject, &obj))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->RenderedThisFrame(p,obj);
		PyObject *ret = (p) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}


} // namespace PyNumbra


