/******************************************************************************
 *    PyNumbraModule.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/links.h>
#include <lg/objects.h>
#include <lg/scrmanagers.h>
#include "interface.h"

#include "PyNumbraModule.h"

#include "PyNumbraExceptions.h"

#include "ScriptInterface.h"
#include "ScriptServices.h"

#include "PyNumbraTypes.h"

namespace PyNumbra
{

CInterface<ILinkManager> CLinkManager;
CInterface<IObjectSystem> CObjectSystem;
CInterface<ITraitManager> CTraitManager;

PyObject * RelationCache::m_Relations = NULL;

PyObject * RelationCache::m_RelationMap = NULL;

void RelationCache::Init(void)
{
	ILinkManager *pLM;
	try
	{
		pLM = CLinkManager.Acquire();
	}
	catch (no_interface &err)
	{
		Exception::Raise(Exception::NoInterfaceError, "RelationCache::Init");
		return;
	}
	short relCount = 0;
	IRelation *pRel = NULL;
	{
		sRelationIter relIter;
		short relId;
		pLM->IterStart(&relIter);
		while (pLM->IterNext(&relIter, &relId))
		{
			if (relId > relCount)
				relCount = relId;
		}
	}
	if (relCount)
	{
		m_Relations = PyList_New(relCount);
		m_RelationMap = PyDict_New();
		//while (relCount)
		for (short relId = 1; relId <= relCount; ++relId)
		{
			pRel = pLM->GetRelation(relId);
			const sRelationDesc *pDesc = pRel->Describe();
			PyObject *obName = PyString_FromString(pDesc->szName);
			PyObject *obNum = PyInt_FromLong(relId);
			PyDict_SetItem(m_RelationMap, obName, obNum);
			Py_DECREF(obNum);
			PyList_SET_ITEM(m_Relations, relId - 1, obName);
			pRel->Release();
		}
	}
	CLinkManager.Release();
}

int RelationCache::Named(PyObject *obName)
{
	if (!m_RelationMap)
		return 0;
	PyObject *obID = PyDict_GetItem(m_RelationMap, obName);
	if (!obID)
		return 0;
	int id = PyInt_AsLong(obID);
	return id;
}

PyObject * RelationCache::Name(int id)
{
	PyObject *obRet = Py_None;
	if (m_Relations && id < PyList_GET_SIZE(m_Relations))
	{
		obRet = PyList_GET_ITEM(m_Relations, id - 1);
	}
	Py_INCREF(obRet);
	return obRet;
}


PyObject * LinkKindNamed(PyObject *, PyObject *arg)
{
	if (!PyObject_TypeCheck(arg, &PyBaseString_Type))
	{
		PyErr_SetString(PyExc_TypeError, "RelationNamed requires a string argument.");
		return NULL;
	}
	return PyInt_FromLong(RelationCache::Named(arg));
}

PyObject * LinkKindName(PyObject *, PyObject *arg)
{
	PyObject *obID = PyNumber_Int(arg);
	if (!obID)
	{
		PyErr_SetString(PyExc_TypeError, "RelationName requires an integer argument.");
		return NULL;
	}
	PyObject *ret = RelationCache::Name(PyInt_AsLong(obID));
	Py_DECREF(obID);
	return ret;
}


static const char * MainModuleDocStrings[] = {
	"PyNumbra imbedded Module for Python\n\n",
	"LinkKindNamed(str) -> Integer\n\t"
		"Returns the relation ID for the given name, or 0 if it doesn't exist.\n\t"
		"The relation names are cached to make this function much faster than\n\t"
		"making an interface call.",
	"LinkKindName(int) -> String\n\t"
		"Returns the relation name for the given ID, or None if it doesn't exist.\n\t"
		"The relation names are cached to make this function much faster than\n\t"
		"making an interface call.",
};

static const PyMethodDef MainModuleMethods[] = {
	{ "LinkKindNamed", LinkKindNamed, METH_O, const_cast<char*>(MainModuleDocStrings[1]) },
	{ "LinkKindName", LinkKindName, METH_O, const_cast<char*>(MainModuleDocStrings[2]) },
	
	{ "_construct_object", reinterpret_cast<PyCFunction>(ObjectObject::Construct),
		METH_VARARGS | METH_KEYWORDS, NULL },
	{ "_pickle_object", reinterpret_cast<PyCFunction>(ObjectObject::Pickle),
		METH_O, NULL },
	{ "_construct_link", reinterpret_cast<PyCFunction>(LinkObject::Construct),
		METH_VARARGS | METH_KEYWORDS, NULL },
	{ "_pickle_link", reinterpret_cast<PyCFunction>(LinkObject::Pickle),
		METH_O, NULL },
	{ "_construct_timer", reinterpret_cast<PyCFunction>(TimerObject::Construct),
		METH_VARARGS | METH_KEYWORDS, NULL },
	{ "_pickle_timer", reinterpret_cast<PyCFunction>(TimerObject::Pickle),
		METH_O, NULL },
	{ NULL, NULL, 0, NULL }
};

extern "C" __declspec(dllexport)
PyObject * initPyNumbra(void)
{
	PyObject *module = Py_InitModule3("PyNumbra", 
					const_cast<PyMethodDef*>(MainModuleMethods),
					const_cast<char*>(MainModuleDocStrings[0]));
	if (module)
	{
		Exception::InitModule(module);

		RelationCache::Init();
		if (PyErr_Occurred())
		{
			MonoException("RelationCache::Init");
			PyErr_Clear();
		}
		ObjectObject::InitModule(module);
		if (PyErr_Occurred())
		{
			MonoException("ObjectObject::InitModule");
			PyErr_Clear();
		}
		LinkObject::InitModule(module);
		if (PyErr_Occurred())
		{
			MonoException("LinkObject::InitModule");
			PyErr_Clear();
		}
		LinkIterObject::InitModule(module);
		if (PyErr_Occurred())
		{
			MonoException("LinkIterObject::InitModule");
			PyErr_Clear();
		}
		TimerObject::InitModule(module);
		if (PyErr_Occurred())
		{
			MonoException("TimerObject::InitModule");
			PyErr_Clear();
		}

		PyObject *dir = PyModule_GetDict(module);
		PyObject *submod = initScriptInterface();
		if (submod)
		{
			PyDict_SetItemString(dir, "ScriptInterface", submod);
		}
		if (PyErr_Occurred())
		{
			MonoException("initScriptInterface");
			PyErr_Clear();
		}
		submod = initScriptServices();
		if (submod)
		{
			PyDict_SetItemString(dir, "ScriptServices", submod);
		}
		if (PyErr_Occurred())
		{
			MonoException("initScriptServices");
			PyErr_Clear();
		}
	}
	return module;
}

} // namespace PyNumbra
