/******************************************************************************
 *    QuestService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IQuestSrv>::Name = "QuestService";

	const char * 
	ScriptService<IQuestSrv>::DocStrings[] = {
		"QuestService()\n\t"
			"Allows you to get and set the values of quest variables.",
		"SubscribeMsg(obj,name,flags) -> Integer\n\t"
			"Begin listening for changes in the given quest variable.\n\t"
			"Only works with mission variables.",
		"UnsubscribeMsg(obj,name) -> Integer\n\t"
			"Stop listening to the quest variable.",
		"Set(name,value,type) -> Integer\n\t"
			"Modifies the quest variable. If it doesn't already exist,\n\t"
			"it will be created in the database specified.",
		"Get(name) -> Integer\n\t"
			"Retrieves the value of a quest variable.",
		"Exists(name) -> Boolean\n\t"
			"Checks if the quest variable exists in any database.",
		"Delete(name) -> Integer\n\t"
			"Removes the quest variable from the database.",
	};

	const PyMethodDef 
	ScriptService<IQuestSrv>::Methods[] = {
		{	"SubscribeMsg",
			reinterpret_cast<PyCFunction>(SubscribeMsg),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"UnsubscribeMsg",
			reinterpret_cast<PyCFunction>(UnsubscribeMsg),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"Set",
			reinterpret_cast<PyCFunction>(Set),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"Get",
			reinterpret_cast<PyCFunction>(Get),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"Exists",
			reinterpret_cast<PyCFunction>(Exists),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"Delete",
			reinterpret_cast<PyCFunction>(Delete),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IQuestSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IQuestSrv>::Name),
		sizeof(ScriptServiceObject<IQuestSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IQuestSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IQuestSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<IQuestSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kMissionQVar", 0);
		PyModule_AddIntConstant(module, "kCampaignQVar", 1);
		// this is wrong
		PyModule_AddIntConstant(module, "kAnyQVar", 2);
	}

	ScriptServiceObject<IQuestSrv> * 
	ScriptService<IQuestSrv>::NewObject(PyObject *)
	{
		IQuestSrv *iface = static_cast<IQuestSrv*>(g_pScriptManager->GetService(IID_IQuestScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IQuestSrv> *obj = PyObject_New(ScriptServiceObject<IQuestSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IQuestSrv>::SubscribeMsg(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		int type;
		object obj;
		if (!PyArg_ParseTuple(args, "O&si:SubscribeMsg", cvtObject, &obj, &name, &type))
		{
			return NULL;
		}
		int ret = self->m_pIFace->SubscribeMsg(obj,name,type);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IQuestSrv>::UnsubscribeMsg(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		if (!PyArg_ParseTuple(args, "O&s:UnsubscribeMsg", cvtObject, &obj, &name))
		{
			return NULL;
		}
		int ret = self->m_pIFace->UnsubscribeMsg(obj,name);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IQuestSrv>::Set(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		int type;
		int value;
		if (!PyArg_ParseTuple(args, "sii:Set", &name, &value, &type))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Set(name,value,type);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IQuestSrv>::Get(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:Get", &name))
		{
			return NULL;
		}
		long val = self->m_pIFace->Get(name);
		return PyInt_FromLong(val);
	}

	PyObject * 
	ScriptService<IQuestSrv>::Exists(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:Exists", &name))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->Exists(name)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IQuestSrv>::Delete(ScriptServiceObject<IQuestSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:Delete", &name))
		{
			return NULL;
		}
		int ret = self->m_pIFace->Delete(name);
		return PyInt_FromLong(ret);
	}


} // namespace PyNumbra


