/******************************************************************************
 *    ScriptInterface.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/types.h>
#include <lg/scrmsgs.h>
#include <lg/scrmanagers.h>

#include "pyutils.h"
#include "ScriptMessages.h"
#include "PyNumbraTypes.h"
#include "ScriptInterface.h"

extern IScriptMan *g_pScriptManager;

namespace PyNumbra
{

	PyObject * ScriptInterface::SendMessage(PyObject *, PyObject *args)
	{
		PyObject *msgDict;
		if (!PyArg_ParseTuple(args, "O!:SendMessage", &PyDict_Type, &msgDict))
		{
			return NULL;
		}
		sScrMsg *msg = ScriptMessages::MessageFromObject(msgDict);
		if (!msg)
		{
			return NULL;
		}
		cMultiParm reply;
		g_pScriptManager->SendMessage(msg, static_cast<sMultiParm*>(&reply));
		msg->Release();
		PyObject *ret = MultiParmToObject(reply);
		reply.Free();
		return ret;
	}

	PyObject * ScriptInterface::SendMessage2(PyObject *, PyObject *args)
	{
		const char *msg;
		object src, dest;
		cMultiParm data[3];
		if (!PyArg_ParseTuple(args, "O&O&s|O&O&O&:SendMessage2", cvtObject, &src, cvtObject, &dest,
						&msg, cvtMultiParm, &data[0], cvtMultiParm, &data[1], cvtMultiParm, &data[2]))
		{
			data[0].Free(); data[1].Free(); data[2].Free();
			return NULL;
		}
		cMultiParm reply;
		g_pScriptManager->SendMessage2(reply,src,dest,msg,data[0],data[1],data[2]);
		data[0].Free(); data[1].Free(); data[2].Free();
		PyObject *ret = MultiParmToObject(reply);
		reply.Free();
		return ret;
	}

	PyObject * ScriptInterface::PostMessage(PyObject *, PyObject *args)
	{
		PyObject *msgDict;
		if (!PyArg_ParseTuple(args, "O!:PostMessage", &PyDict_Type, &msgDict))
		{
			return NULL;
		}
		sScrMsg *msg = ScriptMessages::MessageFromObject(msgDict);
		if (!msg)
		{
			return NULL;
		}
		g_pScriptManager->PostMessage(msg);
		msg->Release();
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * ScriptInterface::PostMessage2(PyObject *, PyObject *args)
	{
		const char *msg;
		object src, dest;
		cMultiParm data[3];
		if (!PyArg_ParseTuple(args, "O&O&s|O&O&O&:PostMessage2", cvtObject, &src, cvtObject, &dest,
						&msg, cvtMultiParm, &data[0], cvtMultiParm, &data[1], cvtMultiParm, &data[2]))
		{
			data[0].Free(); data[1].Free(); data[2].Free();
			return NULL;
		}
		// err... I hope this function is smart enough to make a copy of msg.
		g_pScriptManager->PostMessage2(src,dest,msg,data[0],data[1],data[2]);
		data[0].Free(); data[1].Free(); data[2].Free();
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * ScriptInterface::SetTimedMessage(PyObject *, PyObject *args)
	{
		eScrTimedMsgKind type = kSTM_OneShot;
		unsigned long time;
		PyObject *msgDict;
		if (!PyArg_ParseTuple(args, "O!l|i:SetTimedMessage", &PyDict_Type, &msgDict, &time, &type))
		{
			return NULL;
		}
		sScrMsg *msg = ScriptMessages::MessageFromObject(msgDict);
		if (!msg)
		{
			return NULL;
		}
		g_pScriptManager->SetTimedMessage(msg,time,type);
		msg->Release();
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * ScriptInterface::SetTimedMessage2(PyObject *, PyObject *args)
	{
		eScrTimedMsgKind type = kSTM_OneShot;
		unsigned long time;
		const char *name;
		object dest;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "O&sl|iO&:SetTimedMessage2", cvtObject, &dest, &name, &time, &type, cvtMultiParm, &data))
		{
			return NULL;
		}
		tScrTimer handle = g_pScriptManager->SetTimedMessage2(dest,name,time,type,data);
		data.Free();
		return TimerObject::New(handle);
	}

	PyObject * ScriptInterface::KillTimedMessage(PyObject *, PyObject *args)
	{
		tScrTimer timer;
		if (!PyArg_ParseTuple(args, "l:KillTimedMessage", &timer))
		{
			return NULL;
		}
		g_pScriptManager->KillTimedMessage(timer);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * ScriptInterface::IsScriptDataSet(PyObject *, PyObject *args)
	{
		PyObject *script;
		PyObject *obName = NULL;
		sScrDatumTag tag;
		if (!PyArg_ParseTuple(args, "OO&s:IsScriptDataSet", &script, cvtObject, &tag.objId, &tag.pszName))
		{
			return NULL;
		}
		if (PyInstance_Check(script) || PyClass_Check(script))
		{
			obName = GetObjectName(script);
			if (!obName)
			{
				PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
				return NULL;
			}
			tag.pszClass = PyString_AsString(obName);
		}
		else if (PyString_CheckExact(script))
		{
			tag.pszClass = PyString_AsString(script);
		}
		else if (PyObject_TypeCheck(script, &PyBaseString_Type))
		{
			obName = PyObject_Str(script);
			tag.pszClass = PyString_AsString(obName);
		}
		else
		{
			PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
			return NULL;
		}
		PyObject *ret = (g_pScriptManager->IsScriptDataSet(&tag)) ? Py_True : Py_False;
		Py_INCREF(ret);
		Py_XDECREF(obName);
		return ret;
	}

	PyObject * ScriptInterface::GetScriptData(PyObject *, PyObject *args)
	{
		PyObject *script;
		PyObject *obName = NULL;
		sScrDatumTag tag;
		if (!PyArg_ParseTuple(args, "OO&s:GetScriptData", &script, cvtObject, &tag.objId, &tag.pszName))
		{
			return NULL;
		}
		if (PyInstance_Check(script) || PyClass_Check(script))
		{
			obName = GetObjectName(script);
			if (!obName)
			{
				PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
				return NULL;
			}
			tag.pszClass = PyString_AsString(obName);
		}
		else if (PyString_CheckExact(script))
		{
			tag.pszClass = PyString_AsString(script);
		}
		else if (PyObject_TypeCheck(script, &PyBaseString_Type))
		{
			obName = PyObject_Str(script);
			tag.pszClass = PyString_AsString(obName);
		}
		else
		{
			PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
			return NULL;
		}
		cMultiParm data;
		g_pScriptManager->GetScriptData(&tag, static_cast<sMultiParm*>(&data));
		PyObject *ret = MultiParmToObject(data);
		data.Free();
		Py_XDECREF(obName);
		return ret;
	}

	PyObject * ScriptInterface::SetScriptData(PyObject *, PyObject *args)
	{
		PyObject *script;
		PyObject *obName = NULL;
		sScrDatumTag tag;
		cMultiParm data;
		if (!PyArg_ParseTuple(args, "OO&sO&:SetScriptData", &script, cvtObject, &tag.objId, &tag.pszName, cvtMultiParm, &data))
		{
			data.Free();
			return NULL;
		}
		if (PyInstance_Check(script) || PyClass_Check(script))
		{
			obName = GetObjectName(script);
			if (!obName)
			{
				PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
				data.Free();
				return NULL;
			}
			tag.pszClass = PyString_AsString(obName);
		}
		else if (PyString_CheckExact(script))
		{
			tag.pszClass = PyString_AsString(script);
		}
		else if (PyObject_TypeCheck(script, &PyBaseString_Type))
		{
			obName = PyObject_Str(script);
			tag.pszClass = PyString_AsString(obName);
		}
		else
		{
			PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
			data.Free();
			return NULL;
		}
		long ret = g_pScriptManager->SetScriptData(&tag,static_cast<sMultiParm*>(&data));
		data.Free();
		Py_XDECREF(obName);
		return PyInt_FromLong(ret);
	}

	PyObject * ScriptInterface::ClearScriptData(PyObject *, PyObject *args)
	{
		PyObject *script;
		PyObject *obName = NULL;
		sScrDatumTag tag;
		if (!PyArg_ParseTuple(args, "OO&s:ClearScriptData", &script, cvtObject, &tag.objId, &tag.pszName))
		{
			return NULL;
		}
		if (PyInstance_Check(script) || PyClass_Check(script))
		{
			obName = GetObjectName(script);
			if (!obName)
			{
				PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
				return NULL;
			}
			tag.pszClass = PyString_AsString(obName);
		}
		else if (PyString_CheckExact(script))
		{
			tag.pszClass = PyString_AsString(script);
		}
		else if (PyObject_TypeCheck(script, &PyBaseString_Type))
		{
			obName = PyObject_Str(script);
			tag.pszClass = PyString_AsString(obName);
		}
		else
		{
			PyErr_SetString(PyExc_TypeError, "argument must be a class object or instance, or a string.");
			return NULL;
		}
		cMultiParm data;
		long ret = g_pScriptManager->ClearScriptData(&tag, static_cast<sMultiParm*>(&data));
		data.Free();
		Py_XDECREF(obName);
		return PyInt_FromLong(ret);
	}


	const PyMethodDef ScriptInterface::Methods[] = {
		{ "SendMessage", SendMessage, METH_VARARGS, const_cast<char*>(DocStrings[1]) },
		{ "SendMessage2", SendMessage2, METH_VARARGS, const_cast<char*>(DocStrings[2]) },
		{ "PostMessage", PostMessage, METH_VARARGS, const_cast<char*>(DocStrings[3]) },
		{ "PostMessage2", PostMessage2, METH_VARARGS, const_cast<char*>(DocStrings[4]) },
		{ "SetTimedMessage", SetTimedMessage, METH_VARARGS, const_cast<char*>(DocStrings[5]) },
		{ "SetTimedMessage2", SetTimedMessage2, METH_VARARGS, const_cast<char*>(DocStrings[6]) },
		{ "KillTimedMessage", KillTimedMessage, METH_VARARGS, const_cast<char*>(DocStrings[7]) },
		{ "IsScriptDataSet", IsScriptDataSet, METH_VARARGS, const_cast<char*>(DocStrings[8]) },
		{ "GetScriptData", GetScriptData, METH_VARARGS, const_cast<char*>(DocStrings[9]) },
		{ "SetScriptData", SetScriptData, METH_VARARGS, const_cast<char*>(DocStrings[10]) },
		{ "ClearScriptData", ClearScriptData, METH_VARARGS, const_cast<char*>(DocStrings[11]) },
		{ NULL, NULL, 0, NULL }
	};

	const char * ScriptInterface::DocStrings[] = {
		"PyNumbra Script Interface",
		"SendMessage(msg)\n\t"
			"Accepts a dictionary with the fields of the message. Returns the message\n\t"
			"reply. Raises TypeError if msg is not a dictionary, or ValueError if the\n\t"
			"fields are wrong.",
		"SendMessage2(src,dest,name,data = None,data2 = None,data3 = None)\n\t"
			"src and dst are integers (or objects), name is a string. Returns the\n\t"
			"message reply. Raises TypeError.",
		"PostMessage(msg)\n\t"
			"Accepts a dictionary with the fields of the message. Returns immediately\n\t"
			"with None. Raises TypeError if msg is not a dictionary, or ValueError if\n\t"
			"the fields are wrong.",
		"PostMessage2(src,dest,name,data = None,data2 = None,data3 = None)\n\t"
			"src and dst are integers (or objects), name is a string. Returns None.\n\t"
			"Raises TypeError.",
		"SetTimedMessage(msg,time,type = kSTM_OneShot)\n\t"
			"msg is as SendMessage. time and type are integers. Returns None.",
		"SetTimedMessage2(dest,name,time,type = kSTM_OneShot,data = None)\n\t"
			"Returns a timer object.",
		"KillTimedMessage(timer)\n\t"
			"Takes an integer or timer object. Returns None.",
		"IsScriptDataSet(script,id,name)\n\t"
			"id is an integer or object, name is a string, script is a class or\n\t"
			"instance object. Returns 0 or 1.",
		"GetScriptData(script,id,name)\n\t"
			"Returns an integer, float, string, 3-tuple, or None.",
		"SetScriptData(script,id,name,data)\n\t"
			"script is the script instance or class. It can also be a string.\n\t"
			"data is an integer, float, string, boolean, or 3-tuple.",
		"ClearScriptData(script,id,name)\n\t"
			"The name of the script will be extracted from the instance or\n\t"
			"class object. You can still pass a string to be explicit.\n\t"
			"id is the object id. name is the variable name.",
	};

	extern "C" PyObject *
	initScriptInterface(void)
	{
		PyObject *module = Py_InitModule3("PyNumbra.ScriptInterface", 
					const_cast<PyMethodDef*>(ScriptInterface::Methods), 
					const_cast<char*>(ScriptInterface::DocStrings[0]));
		if (module)
		{
			PyModule_AddIntConstant(module, "kSTM_OneShot", kSTM_OneShot);
			PyModule_AddIntConstant(module, "kSTM_Periodic", kSTM_Periodic);

			PyModule_AddIntConstant(module, "kDoorActionOpen", sDoorMsg::kOpen);
			PyModule_AddIntConstant(module, "kDoorActionClose", sDoorMsg::kClose);
			PyModule_AddIntConstant(module, "kDoorActionOpening", sDoorMsg::kOpening);
			PyModule_AddIntConstant(module, "kDoorActionClosing", sDoorMsg::kClosing);
			PyModule_AddIntConstant(module, "kDoorActionHalt", sDoorMsg::kHalt);

			PyModule_AddIntConstant(module, "kBodyMotionStart", sBodyMsg::kMotionStart);
			PyModule_AddIntConstant(module, "kBodyMotionEnd", sBodyMsg::kMotionEnd);
			PyModule_AddIntConstant(module, "kBodyMotionFlagReached", sBodyMsg::kMotionFlagReached);

			PyModule_AddIntConstant(module, "kRoomEnter", sRoomMsg::kEnter);
			PyModule_AddIntConstant(module, "kRoomExit", sRoomMsg::kExit);
			PyModule_AddIntConstant(module, "kRoomTransit", sRoomMsg::kRoomTransit);
			PyModule_AddIntConstant(module, "kRoomPlayer", sRoomMsg::kPlayer);
			PyModule_AddIntConstant(module, "kRoomRemotePlayer", sRoomMsg::kRemotePlayer);
			PyModule_AddIntConstant(module, "kRoomCreature", sRoomMsg::kCreature);
			PyModule_AddIntConstant(module, "kRoomObject", sRoomMsg::kObject);
			PyModule_AddIntConstant(module, "kRoomNull", sRoomMsg::kNull);
		}
		return module;
	}


} // namespace PyNumbra

