/******************************************************************************
 *    ScriptInterpreter.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <string>
#include <map>
#include <vector>
#include <cstring>
#include <cstdio>
#include <io.h>
#include <lg/types.h>

#include "ScriptInterpreter.h"

#include "PyNumbraExceptions.h"
#include "pyutils.h"
#include "PyNumbraModule.h"
#include "ScriptMessages.h"
#include "ScriptPersistence.h"
#ifdef MONOOBJECT
#include "MonologObject.h"
#endif

namespace PyNumbra
{
	using namespace std;

	const char * CScriptInterpreter::g_pNull = NULL;

	CScriptInterpreter::~CScriptInterpreter()
	{
		if (Py_IsInitialized())
			Finalize();
		
		if (m_pScriptNames)
		{
			delete[] *m_pScriptNames;
			delete[] m_pScriptNames;
		}

		if (m_pScriptPath != &g_pNull)
		{
			delete[] *m_pScriptPath;
			delete[] m_pScriptPath;
		}

		if (m_pPythonHome)
		{
			delete[] m_pPythonHome;
		}
	}

	CScriptInterpreter::CScriptInterpreter()
		: m_iRefCount(1), m_bInitialized(false), m_pPythonHome(NULL), 
		  m_pScriptNames(NULL), m_iScriptPathSize(0), 
		  m_iLoadCount(0), m_bEvalLock(false)
	{
		m_pScriptPath = const_cast<char **>(&g_pNull);
		PyEval_InitThreads();
	}

	unsigned long CScriptInterpreter::AddRef()
	{
		return ++m_iRefCount;
	}

	unsigned long CScriptInterpreter::Release()
	{
		if (--m_iRefCount == 0)
		{
			delete this;
		}
		return m_iRefCount;
	}

	HRESULT CScriptInterpreter::QueryInterface(REFIID riid, void **ppout)
	{
		if (riid == IID_IUnknown)
			*ppout = this;
		else
			return E_NOINTERFACE;
		static_cast<IUnknown*>(*ppout)->AddRef();
		return S_OK;
	}

	void CScriptInterpreter::SetPythonHome(const char *pszPath)
	{
		char *home = new(nothrow) char[strlen(pszPath)];
		if (home)
		{
			strcpy(home,pszPath);
			if (m_pPythonHome)
				delete[] m_pPythonHome;
			m_pPythonHome = home;
			Py_SetPythonHome(home);
		}
	}

	int CScriptInterpreter::SetScriptPath(const char *pszPath)
	{
		DEBUGPRINTF("=== Setting PyNumbra search path to %s\n", pszPath);
		const char *pFirst = pszPath;
		const char *pLast = pszPath + strlen(pszPath) - 1;
		while (strchr("+; \t\r\n", *pFirst)) ++pFirst;
		while (pLast > pFirst && strchr("+; \t\r\n", *pLast)) --pLast;
		if (pFirst == pLast)
			return -1;
		++pLast;
		char *pszCopy = new(nothrow) char[(pLast - pFirst) + 1];
		if (!pszCopy)
			return -1;
		strncpy(pszCopy, pFirst, pLast - pFirst);
		pszCopy[pLast - pFirst] = '\0';
		unsigned int count = strtokcount(pszCopy, "+; \t\r\n");
		char **pNewPath = new(nothrow) char*[count + 1];
		if (!pNewPath)
		{
			delete[] pszCopy;
			return -1;
		}
		count = 0;
		char *pszTok, *pszSep;
		for (pszSep = pszCopy, pszTok = strsep(&pszSep, "+; \t\r\n");
			 pszTok; pszTok = strsep(&pszSep, "+; \t\r\n"))
		{
			if (*pszTok)
				pNewPath[count++] = pszTok;
		}
		pNewPath[count] = NULL;
		if (m_pScriptPath != &g_pNull)
		{
			delete[] *m_pScriptPath;
			delete[] m_pScriptPath;
		}
		if (m_pScriptNames)
		{
			delete[] *m_pScriptNames;
			delete[] m_pScriptNames;
			m_pScriptNames = NULL;
		}
		m_ScriptFiles.clear();
		m_pScriptPath = pNewPath;
		m_iScriptPathSize = count;

		return count;
	}

	const char * const * CScriptInterpreter::GetScriptNames(void)
	{
		DEBUGPRINTF("=== Rebuilding pyscript name cache...\n");
		if (m_pScriptNames)
		{
			delete[] *m_pScriptNames;
			delete[] m_pScriptNames;
			m_pScriptNames = NULL;
		}
		m_ScriptFiles.clear();

		map<string,const char*> names;
		unsigned long bufsize = 0;
		_finddata_t *ffdata = new _finddata_t;
		for (char **pPath = m_pScriptPath; *pPath; ++pPath)
		{
			char fileSpec[MAX_PATH];
			strncpy(fileSpec, *pPath, MAX_PATH);
			strncat(fileSpec, "\\Scripts\\*.py", MAX_PATH);
			int hFFile = _findfirst(fileSpec, ffdata);
			if (hFFile != -1)
			{
				do
				{
					if (ffdata->name[0] != '_')
					{
						string fname = string(ffdata->name);
						fname.resize(fname.size() - 3);
						bufsize += fname.size() + 1;
						names.insert(make_pair(fname,*pPath));
					}
				}
				while (_findnext(hFFile, ffdata) != -1);
				_findclose(hFFile);
			}
		}
		delete ffdata;

		char *pBuf = NULL;
		// This buffer will hold the names of all the scripts.
		// pNames[0] == pBuf
		if (bufsize)
			pBuf = new char[bufsize];
		char ** pNames = new char*[names.size() + 1];
		unsigned int n = 0;
		map<string,const char*>::const_iterator p;
		for (p = names.begin(); p != names.end(); ++p)
		{
			p->first.copy(pBuf, p->first.size());
			pBuf[p->first.size()] = '\0';
			pNames[n++] = pBuf;
			m_ScriptFiles.insert(make_pair(pBuf,p->second));
			pBuf += p->first.size() + 1;
		}
		pNames[n] = NULL;
		m_pScriptNames = pNames;

		return m_pScriptNames;
	}

	void CScriptInterpreter::InvalidateScriptCache(void)
	{
		_classmap_type::iterator cachedclass = m_Classes.begin();
		for (; cachedclass != m_Classes.end(); ++cachedclass)
		{
			Py_DECREF(cachedclass->second);
		}
		m_Classes.clear();
	}

	PyObject * CScriptInterpreter::LoadScript(const char *pszName)
	{
		if (!Load())
			return NULL;

		_classmap_type::const_iterator cachedclass = m_Classes.find(pszName);
		if (cachedclass != m_Classes.end())
		{
			PyObject *obClass = cachedclass->second;
			Py_INCREF(obClass);
			return obClass;
		}
		if (!m_pScriptNames)
		{
			GetScriptNames();
		}
		_filemap_type::const_iterator cachedfile = m_ScriptFiles.find(pszName);
		if (cachedfile == m_ScriptFiles.end())
		{
			Unload();
			return NULL;
		}
		string sModuleName("Scripts.");
		sModuleName += pszName;
		PyObject *obClass = NULL;
		PyObject *obModule = PyImport_ImportModule(const_cast<char*>(sModuleName.c_str()));
		if (obModule)
		{
			obClass = PyDict_GetItemString(PyModule_GetDict(obModule), const_cast<char*>(pszName));
			Py_XINCREF(obClass);
			Py_DECREF(obModule);
		}
		if (PyErr_Occurred())
		{
			MonoException("LoadScript");
			PyErr_Clear();
		}
		if (!obClass)
		{
			Unload();
			return NULL;
		}
		if (!PyClass_Check(obClass))
		{
			Py_DECREF(obClass);
			Unload();
			return NULL;
		}
		m_Classes.insert(make_pair(pszName, obClass));

		Py_INCREF(obClass);
		return obClass;
	}

	void CScriptInterpreter::UnloadScript(const char *pszName, PyObject *pClass)
	{
		if (!m_bInitialized)
			return;
		
		// You should *only* call UnloadScript with a valid handle received
		// from LoadScript
		Py_DECREF(pClass);
		Unload();
	}

	int CScriptInterpreter::Initialize(void)
	{
		Py_Initialize();
		if (!Py_IsInitialized())
		{
			g_pfnMPrintf("!!! Could not initialize Python interpreter!\n");
			m_bInitialized = false;
			return -1;
		}
		if (PyErr_Occurred())
		{
			MonoException("Py_Initialize", true);
			PyErr_Clear();
		}
#if (defined MONOTHREAD) || (defined MONOOBJECT)
		PyObject *obMono = OpenMonolog();
		if (obMono)
		{
			PySys_SetObject("stdout", obMono);
			PySys_SetObject("stderr", obMono);
			Py_DECREF(obMono);
		}

		if (PyErr_Occurred())
		{
			MonoException("Initialize", true);
			PyErr_Clear();
		}
#endif
		
		if (!initPyNumbra())
		{
			MonoException("initPyNumbra");
			Py_Finalize();
			m_bInitialized = false;
			return -1;
		}

		if (PyErr_Occurred())
		{
			MonoException("initPyNumbra");
			PyErr_Clear();
		}

		PyObject *obSysPath = PySys_GetObject("path");
		int index = 0;
		for (char **pPath = m_pScriptPath; *pPath; ++pPath)
		{
			PyObject *obPathString = PyString_FromString(*pPath);
			PyList_Insert(obSysPath,index++,obPathString);
			Py_DECREF(obPathString);
		}

		if (PyErr_Occurred())
		{
			MonoException("Initialize");
			PyErr_Clear();
		}

		m_bInitialized = true;
		return 0;
	}

	void CScriptInterpreter::Finalize(void)
	{
		_classmap_type::iterator cachedclass = m_Classes.begin();
		for (; cachedclass != m_Classes.end(); ++cachedclass)
		{
			Py_DECREF(cachedclass->second);
		}
		m_Classes.clear();
		DEBUGPRINTF("--- Interpreter going bye-bye.\n\n");
		Py_Finalize();
		m_bInitialized = false;
		m_bEvalLock = false;
	}

	unsigned long CScriptInterpreter::Load(void)
	{
		if (!Py_IsInitialized())
			if (0 != Initialize())
				return 0;
		return ++m_iLoadCount;
	}

	void CScriptInterpreter::Unload(void)
	{
		if (--m_iLoadCount == 0)
		{
			Finalize();
		}
	}

	PyObject * CScriptInterpreter::BeginScript(const char *pszName, PyObject *pClass, int iObjId)
	{
		if (!m_bInitialized)
			return NULL;
		
		PyObject *obScript = NULL;
		PyObject *obPickleString = GetScriptPersistence(pClass, iObjId);
		if (obPickleString)
		{
			obScript = pickleReconstruct(obPickleString);
			Py_DECREF(obPickleString);
		}
		if (!obScript)
		{
			PyObject *args = Py_BuildValue("(i)", iObjId);
			if (!args)
			{
				return NULL;
			}
			obScript = PyInstance_New(pClass, args, NULL);

			Py_DECREF(args);
		}

		if (PyErr_Occurred())
		{
			MonoException("BeginScript");
			PyErr_Clear();
		}
		return obScript;
	}

	void CScriptInterpreter::EndScript(PyObject *pScript, PyObject *pClass, int iObjId)
	{
		if (!m_bInitialized)
			return;
		
		if (PyObject_HasAttrString(pClass, "__getstate__"))
		{
			PyObject *obPersistence = pickleDeconstruct(pScript);
			if (obPersistence)
			{
				SetScriptPersistence(obPersistence, pClass, iObjId);
				Py_DECREF(obPersistence);
			}
		}

		if (PyErr_Occurred())
		{
			MonoException("EndScript");
			PyErr_Clear();
		}

		Py_DECREF(pScript);
	}

	int CScriptInterpreter::ReceiveMessage(PyObject *pScript, const sScrMsg *pMsg, sMultiParm *pReply)
	{
		if (!m_bInitialized)
			return 0;
		
		PyObject *obReply = NULL;

		PyObject *obMessage = ScriptMessages::MessageToObject(pMsg);
		if (!obMessage)
			return 0;
		
		if (PyObject_HasAttrString(pScript, "__Messages__"))
		{
			PyObject *obMsgDict = PyObject_GetAttrString(pScript, "__Messages__");
			if (PyDict_Check(obMsgDict))
			{
				PyObject *obMsgName = PyDict_GetItemString(obMessage, "message");
				PyObject *obMsgMethod = PyDict_GetItem(obMsgDict, obMsgName);
				if (obMsgMethod && PyCallable_Check(obMsgMethod))
				{
					obReply = PyObject_CallFunction(obMsgMethod, "OO", pScript, obMessage);
				}
			}
			Py_DECREF(obMsgDict);
		}
		if (!obReply && !PyErr_Occurred())
		{
			if (PyObject_HasAttrString(pScript, "OnMessage"))
			{
				obReply = PyObject_CallMethod(pScript, "OnMessage", "O", obMessage);
			}
		}

		Py_DECREF(obMessage);
		if (obReply)
		{
			if (pReply)
				MultiParmFromObject(static_cast<cMultiParm&>(*pReply), obReply);
			Py_DECREF(obReply);
		}

		if (PyErr_Occurred())
		{
			MonoException("ReceiveMessage");
			PyErr_Clear();
		}

		return 1;
	}

	int CScriptInterpreter::Eval(const char *pszCmd)
	{
		// If Eval is called once, it'll likely be called many more times
		// So the interpreter is forced into a permanent open. 
		// At least, until the destructor is called. Or I add another method to
		// end an Eval session.
		if (!m_bEvalLock)
		{
			if (!Load())
				return 0;
			m_bEvalLock = true;
		}

		PyObject *obMain = PyImport_AddModule("__main__");
		if (!obMain)
		{
			MonoException("py_eval");
			PyErr_Clear();
			return 0;
		}

		PyObject *obContext = PyModule_GetDict(obMain);

		PyObject *result;
		result = PyRun_String(pszCmd, Py_eval_input, obContext, obContext);

		if (!result)
		{
			MonoException("py_eval");
			PyErr_Clear();
			return 0;
		}

		PyObject *repr = PyObject_Repr(result);
		if (repr)
		{
			MonoOutput(repr, true);
			Py_DECREF(repr);
		}
		else
		{
			MonoException("py_eval");
			PyErr_Clear();
		}

		Py_DECREF(result);
		return 1;
	}

}
