/******************************************************************************
 *    ShockAIService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#if (_DARKGAME == 3)
#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IShockAISrv>::Name = "ShockAIService";

	const char * 
	ScriptService<IShockAISrv>::DocStrings[] = {
		"ShockAIService()\n\t"
			"Only available in System Shock 2",
		"Stun(obj,str,str,time) -> Integer",
		"IsStunned(obj) -> Boolean",
		"UnStun(obj) -> Integer",
		"Freeze(obj,time) -> Integer",
		"UnFreeze(obj) -> Integer",
		"NotifyEnterTripwire(ai,obj)\n\t"
			"Tells an AI that it entered a tripwire, and provides the\n\t"
			"object that the tripwire controls. This is usually a door.",
		"NotifyExitTripwire(ai,obj)\n\t"
			"Tells an AI that it left a tripwire, and provides the\n\t"
			"object that the tripwire controls. This is usually a door.",
		"ObjectLocked(obj) -> Integer\n\t"
			"Returns non-zero if the object is locked.",
		"ValidateSpawn(child,parent)\n\t"
			"Is called when an object (the parent) spawns another object\n\t"
			"(the child).",
	};

	const PyMethodDef 
	ScriptService<IShockAISrv>::Methods[] = {
		{	"Stun",
			reinterpret_cast<PyCFunction>(Stun),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"IsStunned",
			reinterpret_cast<PyCFunction>(IsStunned),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"UnStun",
			reinterpret_cast<PyCFunction>(UnStun),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"Freeze",
			reinterpret_cast<PyCFunction>(Freeze),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"UnFreeze",
			reinterpret_cast<PyCFunction>(UnFreeze),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"NotifyEnterTripwire",
			reinterpret_cast<PyCFunction>(NotifyEnterTripwire),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"NotifyExitTripwire",
			reinterpret_cast<PyCFunction>(NotifyExitTripwire),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"ObjectLocked",
			reinterpret_cast<PyCFunction>(ObjectLocked),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"ValidateSpawn",
			reinterpret_cast<PyCFunction>(ValidateSpawn),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IShockAISrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IShockAISrv>::Name),
		sizeof(ScriptServiceObject<IShockAISrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IShockAISrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IShockAISrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IShockAISrv> * 
	ScriptService<IShockAISrv>::NewObject(PyObject *)
	{
		IShockAISrv *iface = static_cast<IShockAISrv*>(g_pScriptManager->GetService(IID_IShockAIScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IShockAISrv> *obj = PyObject_New(ScriptServiceObject<IShockAISrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IShockAISrv>::Stun(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		const char *s1, *s2;
		object obj;
		float time;
		if (!PyArg_ParseTuple(args, "O&ssf:Stun", cvtObject, &obj, &s1, &s2, &time))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Stun(obj,s1,s2,time));
	}

	PyObject * 
	ScriptService<IShockAISrv>::IsStunned(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:IsStunned", cvtObject, &obj))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsStunned(obj)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IShockAISrv>::UnStun(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:UnStun", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->UnStun(obj));
	}

	PyObject * 
	ScriptService<IShockAISrv>::Freeze(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj;
		float time;
		if (!PyArg_ParseTuple(args, "O&f:Freeze", cvtObject, &obj, &time))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->Freeze(obj,time));
	}

	PyObject * 
	ScriptService<IShockAISrv>::UnFreeze(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:UnFreeze", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->UnFreeze(obj));
	}

	PyObject * 
	ScriptService<IShockAISrv>::NotifyEnterTripwire(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj, trap;
		if (!PyArg_ParseTuple(args, "O&O&:NotifyEnterTripwire", cvtObject, &obj, cvtObject, &trap))
		{
			return NULL;
		}
		self->m_pIFace->NotifyEnterTripwire(obj,trap);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IShockAISrv>::NotifyExitTripwire(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj, trap;
		if (!PyArg_ParseTuple(args, "O&O&:NotifyExitTripwire", cvtObject, &obj, cvtObject, &trap))
		{
			return NULL;
		}
		self->m_pIFace->NotifyExitTripwire(obj,trap);
		Py_INCREF(Py_None);
		return Py_None;
	}

	PyObject * 
	ScriptService<IShockAISrv>::ObjectLocked(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:ObjectLocked", cvtObject, &obj))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->ObjectLocked(obj));
	}

	PyObject * 
	ScriptService<IShockAISrv>::ValidateSpawn(ScriptServiceObject<IShockAISrv> *self, PyObject *args)
	{
		object obj, trap;
		if (!PyArg_ParseTuple(args, "O&O&:ValidateSpawn", cvtObject, &obj, cvtObject, &trap))
		{
			return NULL;
		}
		self->m_pIFace->ValidateSpawn(obj,trap);
		Py_INCREF(Py_None);
		return Py_None;
	}


} // namespace PyNumbra

#endif
