/******************************************************************************
 *    SoundScriptService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<ISoundScrSrv>::Name = "SoundScriptService";

	const char * 
	ScriptService<ISoundScrSrv>::DocStrings[] = {
		"SoundScriptService()\n\t"
			"Ah! I see you have the interface that goes \"Ping!\"\n\t"
			"The SoundNetwork option doesn't exist in Thief 1/G, but\n\t"
			"these functions will accept it anyway.\n\t"
			"The C++ functions are overloaded, so I've tweaked the names\n\t"
			"a bit.",
		"PlayLoc(obj,name,location,special=0,net=0) -> Boolean\n\t"
			"Plays the schema tags at a specific location.",
		"PlayObj(obj,name,src,special=0,net=0) -> Boolean\n\t"
			"Plays the schema tags at the location of another object.",
		"Play(obj,name,special=0,net=0) -> Boolean\n\t"
			"Plays the schema tags at the location of the host object.",
		"PlayAmbient(obj,name,special=0,net=0) -> Boolean\n\t"
			"Plays the schema tags as an environmental ambient.",
		"PlaySchemaLoc(obj,schema,location,net=0) -> Boolean\n\t"
			"Plays a schema object at a specific location.",
		"PlaySchemaObj(obj,schema,src,net=0) -> Boolean\n\t"
			"Plays a schema object at the location of another object.",
		"PlaySchema(obj,schema,net=0) -> Boolean\n\t"
			"Plays a schema object at the location of the host object.",
		"PlaySchemaAmbient(obj,schema,net=0) -> Boolean\n\t"
			"Plays a schema object as an environmental ambient.",
		"PlayEnvSchema(obj,tags,src,src2,loc,net=0) -> Boolean\n\t"
			"Plays the environmental schema tags with arguments.",
		"PlayVoiceOver(obj,schema) -> Boolean\n\t"
			"Plays a schema object as a voice-over.",
		"Halt(obj,name,src) -> Integer\n\t"
			"Stops playing a sound. An empty string will stop all sounds.",
		"HaltSchema(obj,tags,src) -> Boolean\n\t"
			"Stops playing the schema tags. An empty string will stop all\n\t"
			"schemas.",
		"HaltSpeech(ai) -> Integer\n\t"
			"Stops any sounds being made by the ai.",
		"PreLoad(name) -> Boolean\n\t"
			"Caches the audio data of the sound for faster playback later.",
	};

	const PyMethodDef 
	ScriptService<ISoundScrSrv>::Methods[] = {
		{	"PlayLoc",
			reinterpret_cast<PyCFunction>(PlayLoc),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"PlayObj",
			reinterpret_cast<PyCFunction>(PlayObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"Play",
			reinterpret_cast<PyCFunction>(Play),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"PlayAmbient",
			reinterpret_cast<PyCFunction>(PlayAmbient),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"PlaySchemaLoc",
			reinterpret_cast<PyCFunction>(PlaySchemaLoc),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{	"PlaySchemaObj",
			reinterpret_cast<PyCFunction>(PlaySchemaObj),
			METH_VARARGS,
			const_cast<char*>(DocStrings[6])
		},
		{	"PlaySchema",
			reinterpret_cast<PyCFunction>(PlaySchema),
			METH_VARARGS,
			const_cast<char*>(DocStrings[7])
		},
		{	"PlaySchemaAmbient",
			reinterpret_cast<PyCFunction>(PlaySchemaAmbient),
			METH_VARARGS,
			const_cast<char*>(DocStrings[8])
		},
		{	"PlayEnvSchema",
			reinterpret_cast<PyCFunction>(PlayEnvSchema),
			METH_VARARGS,
			const_cast<char*>(DocStrings[9])
		},
		{	"PlayVoiceOver",
			reinterpret_cast<PyCFunction>(PlayVoiceOver),
			METH_VARARGS,
			const_cast<char*>(DocStrings[10])
		},
		{	"Halt",
			reinterpret_cast<PyCFunction>(Halt),
			METH_VARARGS,
			const_cast<char*>(DocStrings[11])
		},
		{	"HaltSchema",
			reinterpret_cast<PyCFunction>(HaltSchema),
			METH_VARARGS,
			const_cast<char*>(DocStrings[12])
		},
		{	"HaltSpeech",
			reinterpret_cast<PyCFunction>(HaltSpeech),
			METH_VARARGS,
			const_cast<char*>(DocStrings[13])
		},
		{	"PreLoad",
			reinterpret_cast<PyCFunction>(PreLoad),
			METH_VARARGS,
			const_cast<char*>(DocStrings[14])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<ISoundScrSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<ISoundScrSrv>::Name),
		sizeof(ScriptServiceObject<ISoundScrSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<ISoundScrSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<ISoundScrSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	void
	ScriptService<ISoundScrSrv>::InitModule(PyObject *module)
	{
		PyType_Ready(&TypeInfo);
		PyModule_AddIntConstant(module, "kSoundNormal", kSoundNormal);
		PyModule_AddIntConstant(module, "kSoundLoop", kSoundLoop);
		PyModule_AddIntConstant(module, "kEnvSoundOnObj", kEnvSoundOnObj);
		PyModule_AddIntConstant(module, "kEnvSoundAtObjLoc", kEnvSoundAtObjLoc);
		PyModule_AddIntConstant(module, "kEnvSoundAmbient", kEnvSoundAmbient);
	}

	ScriptServiceObject<ISoundScrSrv> * 
	ScriptService<ISoundScrSrv>::NewObject(PyObject *)
	{
		ISoundScrSrv *iface = static_cast<ISoundScrSrv*>(g_pScriptManager->GetService(IID_ISoundScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<ISoundScrSrv> *obj = PyObject_New(ScriptServiceObject<ISoundScrSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlayLoc(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		cScrVec loc;
		eSoundSpecial spc = kSoundNormal;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&s(fff)|ii:PlayLoc", cvtObject, &obj, &name,
							&loc.pos.x, &loc.pos.y, &loc.pos.z, &spc, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->Play(p,obj,name,loc,spc);
#else
		self->m_pIFace->Play(p,obj,name,loc,spc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlayObj(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name;
		object obj, src;
		eSoundSpecial spc = kSoundNormal;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&sO&|ii:PlayObj", cvtObject, &obj, &name, cvtObject, &src, &spc, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->Play(p,obj,name,src,spc);
#else
		self->m_pIFace->Play(p,obj,name,src,spc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::Play(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		eSoundSpecial spc = kSoundNormal;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&s|ii:Play", cvtObject, &obj, &name, &spc, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->Play(p,obj,name,spc);
#else
		self->m_pIFace->Play(p,obj,name,spc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlayAmbient(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name;
		object obj;
		eSoundSpecial spc = kSoundNormal;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&s|ii:PlayAmbient", cvtObject, &obj, &name, &spc, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlayAmbient(p,obj,name,spc);
#else
		self->m_pIFace->PlayAmbient(p,obj,name,spc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlaySchemaLoc(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object obj, schema;
		cScrVec loc;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&O&(fff)|i:PlaySchemaLoc", cvtObject, &obj, cvtObject, &schema,
							&loc.pos.x, &loc.pos.y, &loc.pos.z, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlaySchema(p,obj,schema,loc);
#else
		self->m_pIFace->PlaySchema(p,obj,schema,loc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlaySchemaObj(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object obj, schema, src;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&O&O&|i:PlaySchemaObj", cvtObject, &obj, cvtObject, &schema, cvtObject, &src, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlaySchema(p,obj,schema,src);
#else
		self->m_pIFace->PlaySchema(p,obj,schema,src,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlaySchema(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object obj, schema;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&O&|i:PlaySchema", cvtObject, &obj, cvtObject, &schema, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlaySchema(p,obj,schema);
#else
		self->m_pIFace->PlaySchema(p,obj,schema,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlaySchemaAmbient(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object obj, schema;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&O&|i:PlaySchemaAmbient", cvtObject, &obj, cvtObject, &schema, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlaySchemaAmbient(p,obj,schema);
#else
		self->m_pIFace->PlaySchemaAmbient(p,obj,schema,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlayEnvSchema(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *tags;
		object obj, src, src2;
		eEnvSoundLoc loc;
		eSoundNetwork net = kSoundNetwork0;
		if (!PyArg_ParseTuple(args, "O&sO&O&i|i:PlayEnvSchema", cvtObject, &obj, &tags,
							cvtObject, &src, cvtObject, &src2, &loc, &net))
		{
			return NULL;
		}
		true_bool p;
#if (_DARKGAME == 1)
		self->m_pIFace->PlayEnvSchema(p,obj,tags,src,src2,loc);
#else
		self->m_pIFace->PlayEnvSchema(p,obj,tags,src,src2,loc,net);
#endif
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PlayVoiceOver(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object obj, schema;
		if (!PyArg_ParseTuple(args, "O&O&:PlayVoiceOver", cvtObject, &obj, cvtObject, &schema))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->PlayVoiceOver(p,obj,schema);
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::Halt(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name = NULL;
		object obj, src;
		if (!PyArg_ParseTuple(args, "O&zO&:Halt", cvtObject, &obj, &name, cvtObject, &src))
		{
			return NULL;
		}
		int ret = self->m_pIFace->Halt(obj,name,src);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::HaltSchema(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *tags = NULL;
		object obj, src;
		if (!PyArg_ParseTuple(args, "O&zO&:HaltSchema", cvtObject, &obj, &tags, cvtObject, &src))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->HaltSchema(p,obj,tags,src);
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::HaltSpeech(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		object ai;
		if (!PyArg_ParseTuple(args, "O&:HaltSpeech", cvtObject, &ai))
		{
			return NULL;
		}
		long ret = self->m_pIFace->HaltSpeech(ai);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<ISoundScrSrv>::PreLoad(ScriptServiceObject<ISoundScrSrv> *self, PyObject *args)
	{
		const char *name;
		if (!PyArg_ParseTuple(args, "s:PreLoad", &name))
		{
			return NULL;
		}
		true_bool p;
		self->m_pIFace->PreLoad(p,name);
		PyObject *obRet = (p) ? Py_True : Py_False;
		Py_INCREF(obRet);
		return obRet;
	}


} // namespace PyNumbra


