/******************************************************************************
 *    WeaponService.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <lg/iiddef.h>
#include <lg/scrservices.h>
#include <lg/types.h>

#include "ScriptServices.h"
#include "PyNumbraExceptions.h"
#include "pyutils.h"

namespace PyNumbra
{

	const char *
	ScriptService<IWeaponSrv>::Name = "WeaponService";

	const char * 
	ScriptService<IWeaponSrv>::DocStrings[] = {
		"WeaponService()\n\t"
			"Allows the player to wield and use weapons.",
		"Equip(obj,int) -> Integer\n\t"
			"Readies the object. The integer argument is 0 if the weapon will\n\t"
			"be used as a sword, or 1 if it will be used as a blackjack.",
		"UnEquip(obj) -> Integer\n\t"
			"Put away the weapon.",
		"IsEquipped(frobber,obj) -> Boolean\n\t"
			"Check if frobber is wielding obj.",
		"StartAttack(frobber,obj) -> Integer\n\t"
			"Raise the weapon before swinging it.",
		"FinishAttack(frobber,obj) -> Integer\n\t"
			"Swing the weapon.",
	};

	PyMethodDef 
	const ScriptService<IWeaponSrv>::Methods[] = {
		{	"Equip",
			reinterpret_cast<PyCFunction>(Equip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[1])
		},
		{	"UnEquip",
			reinterpret_cast<PyCFunction>(UnEquip),
			METH_VARARGS,
			const_cast<char*>(DocStrings[2])
		},
		{	"IsEquipped",
			reinterpret_cast<PyCFunction>(IsEquipped),
			METH_VARARGS,
			const_cast<char*>(DocStrings[3])
		},
		{	"StartAttack",
			reinterpret_cast<PyCFunction>(StartAttack),
			METH_VARARGS,
			const_cast<char*>(DocStrings[4])
		},
		{	"FinishAttack",
			reinterpret_cast<PyCFunction>(FinishAttack),
			METH_VARARGS,
			const_cast<char*>(DocStrings[5])
		},
		{ NULL, NULL, 0, NULL }
	};

	PyTypeObject 
	ScriptService<IWeaponSrv>::TypeInfo = {
		PyObject_HEAD_INIT(NULL)
		0,
		const_cast<char*>(ScriptService<IWeaponSrv>::Name),
		sizeof(ScriptServiceObject<IWeaponSrv>), 0,
		reinterpret_cast<destructor>(ScriptService<IUnknown>::Dealloc),
		0,
		0, 0,		/* tp_getattr, tp_setattr */
		reinterpret_cast<cmpfunc>(ScriptService<IUnknown>::Compare),
		reinterpret_cast<reprfunc>(ScriptService<IUnknown>::Repr),
		0, 0, 0,	/* tp_as_number, tp_as_sequence, tp_as_mapping */
		0, 0,		/* tp_hash, tp_call */
		0,			/* tp_str */
		0, 0, 0,	/* tp_getattro, tp_setattro, tp_as_buffer */
		Py_TPFLAGS_DEFAULT,		/* tp_flags */
		const_cast<char*>(ScriptService<IWeaponSrv>::DocStrings[0]),
		0, 0, 0,	/* tp_traverse, tp_clear, tp_richcompare */
		0, 0, 0,	/* tp_weaklistoffset, tp_iter, tp_iternext */
		const_cast<PyMethodDef*>(ScriptService<IWeaponSrv>::Methods),
		0,0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,0,0,0,0
	};


	ScriptServiceObject<IWeaponSrv> * 
	ScriptService<IWeaponSrv>::NewObject(PyObject *)
	{
		IWeaponSrv *iface = static_cast<IWeaponSrv*>(g_pScriptManager->GetService(IID_IWeaponScriptService));
		if (!iface)
		{
			Exception::Raise(Exception::NoInterfaceError, Name);
			return NULL;
		}
		ScriptServiceObject<IWeaponSrv> *obj = PyObject_New(ScriptServiceObject<IWeaponSrv>, &TypeInfo);
		if (!obj)
		{
			iface->Release();
			return NULL;
		}
		obj->m_pIFace = iface;
		return obj;
	}

	PyObject * 
	ScriptService<IWeaponSrv>::Equip(ScriptServiceObject<IWeaponSrv> *self, PyObject *args)
	{
		object obj;
		int n;
		if (!PyArg_ParseTuple(args, "O&i:Equip", cvtObject, &obj, &n))
		{
			return NULL;
		}
		long ret = self->m_pIFace->Equip(obj,n);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IWeaponSrv>::UnEquip(ScriptServiceObject<IWeaponSrv> *self, PyObject *args)
	{
		object obj;
		if (!PyArg_ParseTuple(args, "O&:UnEquip", cvtObject, &obj))
		{
			return NULL;
		}
		long ret = self->m_pIFace->UnEquip(obj);
		return PyInt_FromLong(ret);
	}

	PyObject * 
	ScriptService<IWeaponSrv>::IsEquipped(ScriptServiceObject<IWeaponSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:UnEquip", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		PyObject *ret = (self->m_pIFace->IsEquipped(obj1,obj2)) ? Py_True : Py_False;
		Py_INCREF(ret);
		return ret;
	}

	PyObject * 
	ScriptService<IWeaponSrv>::StartAttack(ScriptServiceObject<IWeaponSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:StartAttack", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->StartAttack(obj1,obj2));
	}

	PyObject * 
	ScriptService<IWeaponSrv>::FinishAttack(ScriptServiceObject<IWeaponSrv> *self, PyObject *args)
	{
		object obj1, obj2;
		if (!PyArg_ParseTuple(args, "O&O&:FinishAttack", cvtObject, &obj1, cvtObject, &obj2))
		{
			return NULL;
		}
		return PyInt_FromLong(self->m_pIFace->FinishAttack(obj1,obj2));
	}


} // namespace PyNumbra


