"""Vector arithmetic functions.

    This file is part of PyNumbra
    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

from __future__ import division
from math import sqrt,acos
from operator import add

try:
    sum
except NameError:
    def sum(l):
        return reduce(add, l)
    

__all__ = ['mag', 'dot', 'cross', 'unit', 'angle', 'angle1', 'interpolate']

def mag(v):
    """mag(vector) -> float
    
    Calculates the magnitude of the vector.
    
    sqrt(a1**2 + a2**2 + ...)
    
    """
    
    if type(v) not in (list,tuple):
        return float(v)
    
    m = 0.0
    for a in v:
        m += a ** 2
    return sqrt(m)

def dot(v1,v2):
    """dot(vector,vector) -> float
    
    Calculates the dot product of the vectors.
    
    a1*b1 + a2*b2 + a3*b3 + ...
    
    """
    
    p = 0
    for (a,b) in map(lambda a,b: (float(a or 0),float(b or 0)), v1, v2):
        p += a * b
    return p

def cross(a,b):
    """cross(vector,vector) -> vector
    
    Calculates the cross product of the vectors.
    
    Only works with 3-element vectors, 
    until I find a more general algorithm.
    
    """
    
    if len(a) != 3 or len(b) != 3:
        raise ValueError('cross only works with 3-element vectors, sorry.')
    
    i = a[1]*b[2] - a[2]*b[1]
    j = a[2]*b[0] - a[0]*b[2]
    k = a[0]*b[1] - a[1]*b[0]
    return (i,j,k)

def unit(v):
    """unit(vector) -> vector
    
    Returns the unit vector corresponding to the argument.
    
    v * (1/mag(v))
    
    """
    m = 1.0/mag(v)
    return tuple([m*a for a in v])

def angle(a,b):
    """angle(vector,vector) -> float
    
    Returns the angle between the arguments.
    
    acos(dot(a,b)/(mag(a)*mag(b)))

    """
    d = mag(a)*mag(b)
    if d == 0:
        return 0.0
    t = dot(a,b)/d
    if t > 1.0:
        t = 1.0
    elif t < -1.0:
        t = -1.0
    return acos(t)

def angle1(a,b):
    """angle1(vector,vector) -> float
    
    Returns the angle between the arguments.
    Assumes the arguments are unit-vectors.
    
    acos(dot(a,b))

    """
    t = dot(a,b)
    if t > 1.0:
        t = 1.0
    elif t < -1.0:
        t = -1.0
    return acos(t)

def interpolate(a,b,c):
    """interpolate(vector,vector,scalar) -> vector
    
    a*(1-c) + b*c
    """
    #(a,b,c)*(1-g) + (d,e,f)*g
    return tuple(map(add,[i*(1-c) for i in a],[j*c for j in b]))

