/******************************************************************************
 *    pyutils.cc
 *
 *    This file is part of PyNumbra
 *    Copyright (C) 2004 Tom N Harris <telliamed@whoopdedo.cjb.net>
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *****************************************************************************/

#include <Python.h>
#include "pydefs.h"
#include <new>
#include <cstdarg>
#include <lg/types.h>
#include <lg/scrmanagers.h>
#include <lg/tools.h>

#include <hacks/darkprocs.h>

#include "pyutils.h"
#include "PyNumbraExceptions.h"
#include "PyNumbraTypes.h"

extern IMalloc *g_pMalloc;
extern IScriptMan *g_pScriptManager;
extern mprintf g_pfnMPrintf;
extern StatusField g_pfnStatus;

namespace PyNumbra
{

void monoprintf(const char* pszFormat, ...)
{
	va_list va;
	char *psz;
	int iLen;
	va_start(va, pszFormat);
	iLen = vsprintf(NULL, pszFormat, va);
	va_end(va);
	psz = new(std::nothrow) char[iLen+1];
	if (psz)
	{
		va_start(va, pszFormat);
		vsprintf(psz, pszFormat, va);
		va_end(va);

		g_pfnMPrintf("%.1023s",psz);
		delete[] psz;
	}
}

// Sends repr(ob) to the monolog, 
void MonoOutput(PyObject *ob, bool status)
{
	char *pszRepr;
	int iLen;
	char chBuf[1024];
	if (PyString_AsStringAndSize(ob,&pszRepr,&iLen) < 0)
	{
		PyErr_Clear();
		return;
	}
	char *psz = pszRepr;
	while (iLen > 1022)
	{
		strncpy(chBuf, psz, 1022);
		chBuf[1022] = '\0';
		iLen -= 1022;
		g_pfnMPrintf("%s", chBuf);
		psz += 1022;
	}
	g_pfnMPrintf("%s\n", psz);

	if (status && g_pfnStatus)
	{
		strncpy(chBuf, pszRepr, 128);
		chBuf[128] = '\0';
		g_pfnStatus(4, chBuf);
	}
}

unsigned int strtokcount(const char *str, const char *delim)
{
	register unsigned int count = 1;
	register const char *p = str;
	while ((p = strpbrk(p, delim)))
	{
		++count;
		++p;
	}
	return count;
}

char* strsep(char** str, const char* delim)
{
	register char* p = *str;
	if (!p) return NULL;
	while (*p)
	{
		register const char* d = delim;
		while (*d)
		{
			if (*p == *d++)
			{
				register char* t = *str;
				*p++ = '\0';
				*str = p;
				return t;
			}
		}
		++p;
	}
	p = *str;
	*str = NULL;
	return p;
}

PyObject * GetObjectName(PyObject *ob)
{
	PyObject *ret = NULL;
	if (PyInstance_Check(ob))
	{
		PyObject *obclass = reinterpret_cast<PyObject*>(reinterpret_cast<PyInstanceObject*>(ob)->in_class);
		if (obclass)
		{
			ret = PyObject_GetAttrString(obclass, "__name__");
		}
	}
	else
	{
		ret = PyObject_GetAttrString(ob, "__name__");
		if (!ret)
		{
			PyObject *obtype = PyObject_Type(ob);
			if (obtype)
			{
				ret = PyObject_GetAttrString(obtype, "__name__");
				Py_DECREF(obtype);
			}
		}
	}
	return ret;
}

const char * GetInternString(PyObject *str)
{
	if (!PyObject_TypeCheck(str, &PyBaseString_Type))
	{
		return NULL;
	}
	if (!PyString_CheckExact(str))
	{
		str = PyObject_Str(str);
	}
	else
		Py_INCREF(str);
	PyString_InternInPlace(&str);
	const char *ret = PyString_AsString(str);
	// It used to be that interned strings were never freed.
	// A useful side-effect for sending strings out to the Dark Engine
	// where who-knows-what might happen.
	// Well, that changed in 2.3. So we float a reference to the 
	// string to prevent it from being collected.
	//Py_DECREF(str);
	return ret;
}


int cvtObject(PyObject *ob, object *val)
{
	int i;
	if (PyObject_TypeCheck(ob, &ObjectObject::Type))
	{
		i = static_cast<ObjectObject*>(ob)->m_object;
	}
	else
	{
		PyObject *num = PyNumber_Int(ob);
		if (!num)
		{
			PyErr_SetString(PyExc_TypeError, "argument must be an Object ID");
			return 0;
		}
		i = PyInt_AsLong(num);
		Py_DECREF(num);
	}
	*val = i;
	return 1;
}

int cvtLink(PyObject *ob, link *val)
{
	long i;
	if (PyObject_TypeCheck(ob, &LinkObject::Type))
	{
		i = static_cast<LinkObject*>(ob)->m_link;
	}
	else
	{
		PyObject *num = PyNumber_Int(ob);
		if (!num)
		{
			PyErr_SetString(PyExc_TypeError, "argument must be a Link ID");
			return 0;
		}
		i = PyInt_AsLong(num);
		Py_DECREF(num);
	}
	*val = i;
	return 1;
}

int cvtMultiParm(PyObject *ob, cMultiParm *mp)
{
	if (!MultiParmFromObject(*mp, ob))
	{
		PyErr_SetString(PyExc_TypeError, "argument must be an Integer, Float, String, or Vector");
		return 0;
	}
	return 1;
}

int MultiParmFromObject(cMultiParm &mp, PyObject *ob)
{
	if (PyString_Check(ob) || PyUnicode_Check(ob))
	{
		const char *buf;
		if (0 > PyString_AsStringAndSize(ob, const_cast<char**>(&buf), NULL))
			return 0;
		mp = buf;
		return 1;
	}
	else if (PyObject_TypeCheck(ob, &PyBaseString_Type))
	{
		PyObject *str = PyObject_Str(ob);
		if (!str)
			return 0;
		mp = PyString_AsString(str);
		Py_DECREF(str);
		return 1;
	}
	if (PyInt_Check(ob))
	{
		mp = PyInt_AsLong(ob);
		return 1;
	}
	if (PyFloat_Check(ob))
	{
		mp = PyFloat_AsDouble(ob);
		return 1;
	}
	if (PySequence_Check(ob) && PySequence_Size(ob) == 3)
	{
		mxs_vector v;
		PyObject *tuple = PySequence_Tuple(ob);
		if (!PyArg_ParseTuple(tuple, "fff", &v.x, &v.y, &v.z))
		{
			Py_DECREF(tuple);
			return 0;
		}
		mp = &v;
		Py_DECREF(tuple);
		return 1;
	}
	return 0;
}

PyObject * MultiParmToObject(const cMultiParm& mp)
{
	switch (mp.type)
	{
	case kMT_Undef:
		break;
	case kMT_Int:
		return PyInt_FromLong(mp.i);
	case kMT_Float:
		return PyFloat_FromDouble(mp.f);
	case kMT_String:
		if (mp.psz)
			return PyString_FromString(mp.psz);
		break;
	case kMT_Vector:
		if (mp.pVector)
		{
			return Py_BuildValue("[fff]", mp.pVector->x, mp.pVector->y, mp.pVector->z);
		}
		break;
	case kMT_Boolean:
		{
			PyObject *ret = (mp.b) ? Py_True : Py_False;
			Py_INCREF(ret);
			return ret;
		}
	}
	Py_INCREF(Py_None);
	return Py_None;
}


static PyObject * _ObjectFromField(const sFieldDesc *pField, const char *pData)
{
	PyObject *obRet = NULL;
	char *pElem = const_cast<char*>(pData) + pField->offset;
	switch (pField->type)
	{
		case kFieldTypeInt:
		{
			switch (pField->size)
			{
				case 1:
					obRet = PyInt_FromLong(*pElem); 
					break;
				case 2:
					obRet = PyInt_FromLong(*reinterpret_cast<short*>(pElem)); 
					break;
				default:
					obRet = PyInt_FromLong(*reinterpret_cast<long*>(pElem)); 
					break;
			}
			break;
		}
		case kFieldTypeBoolean:
		{
			int p = 0;
			switch (pField->size)
			{
				case 1:
					p = *pElem; 
					break;
				case 2:
					p = *reinterpret_cast<short*>(pElem); 
					break;
				default:
					p = *reinterpret_cast<long*>(pElem); 
					break;
			}
			obRet = (p) ? Py_True : Py_False;
			Py_INCREF(obRet);
			break;
		}
		case kFieldTypeUnsignedInt:
		// I'm too lazy to map the symbols to numbers. This is faster anyway.
		case kFieldTypeBitVector:
		case kFieldTypeEnum:
		{
			unsigned long l = 0;
			switch (pField->size)
			{
				case 1:
					l = *reinterpret_cast<unsigned char*>(pElem); 
					break;
				case 2:
					l = *reinterpret_cast<unsigned short*>(pElem); 
					break;
				default:
					l = *reinterpret_cast<unsigned long*>(pElem); 
					break;
			}
			if (l > INT_MAX)
			{
				obRet = PyLong_FromUnsignedLong(l);
			}
			else
				obRet = PyInt_FromLong(l);
			break;
		}
		case kFieldTypeString:
		{
			// I don't want to embed NULLs
			char *buf = new(std::nothrow) char[pField->size + 1];
			if (buf)
			{
				strncpy(buf, pElem, pField->size);
				buf[pField->size] = '\0';
				obRet = PyString_FromString(buf);
				delete[] buf;
			}
			break;
		}
		case kFieldTypeData:
		{
			obRet = PyString_FromStringAndSize(pElem, pField->size);
			break;
		}
		case kFieldTypeVector:
		{
			mxs_vector *v = reinterpret_cast<mxs_vector*>(pElem);
			obRet = Py_BuildValue("[fff]", v->x, v->y, v->z);
			break;
		}
		case kFieldTypeFloat:
		{
			if (pField->size > sizeof(float))
				obRet = PyFloat_FromDouble(*reinterpret_cast<double*>(pElem));
			else
				obRet = PyFloat_FromDouble(*reinterpret_cast<float*>(pElem));
			break;
		}
		default:
		{
			obRet = Py_None;
			Py_INCREF(obRet);
			break;
		}
	}
	return obRet;
}

PyObject * StructDataToObject(const char *pszName, const char *pData)
{
	IStructDescTools *pStruct;
	if (E_NOINTERFACE == g_pScriptManager->QueryInterface(IID_IStructDescTools, reinterpret_cast<void**>(&pStruct)))
	{
		Exception::Raise(Exception::NoInterfaceError, "StructDataToObject");
		return NULL;
	}
	const sStructDesc *pSDesc = pStruct->Lookup(pszName);
	if (!pSDesc)
	{
		pStruct->Release();
		PyErr_SetString(PyExc_ValueError, "invalid structure name");
		return NULL;
	}
	if (pStruct->IsSimple(pSDesc))
	{
		PyObject *obRet = _ObjectFromField(&pSDesc->fields[0], pData);
		pStruct->Release();
		return obRet;
	}

	PyObject *obStruct = PyDict_New();
	for (int nField = 0; nField < pSDesc->num_fields; ++nField)
	{
		PyObject *obElem = _ObjectFromField(&pSDesc->fields[nField], pData);
		if (obElem)
		{
			PyDict_SetItemString(obStruct, pSDesc->fields[nField].name, obElem);
			Py_DECREF(obElem);
		}
	}

	pStruct->Release();
	return obStruct;
}

/*
unsigned long StructDataFromObject(const char *pszName, PyObject *obData, char *pData)
{
	IStructDescTools *pStruct;
	if (E_NOINTERFACE == g_pScriptManager->QueryInterface(IID_IStructDescTools, reinterpret_cast<void**>(&pStruct)))
	{
		Exception::Raise(Exception::NoInterfaceError, "StructDataFromObject");
		return NULL;
	}
	const sStructDesc *pSDesc = pStruct->Lookup(pszName);
	if (!pSDesc)
	{
		pStruct->Release();
		PyErr_SetString(PyExc_ValueError, "invalid structure name");
		return NULL;
	}
	if (!pData)
	{
		unsigned long ret = pSDesc->size;
		pStruct->Release();
		return ret;
	}
}
*/


} // namespace PyNumbra
